import os
import urllib2
import anydbm
import shelve
import itertools
import warnings

from functools import wraps
from collections import namedtuple
from operator import itemgetter
from contextlib import closing
from xml.dom import pulldom
from StringIO import StringIO

from Orange.utils import environ


class BioMartError(Exception):
    pass


class BioMartServerError(BioMartError):
    pass


class DatabaseNameConflictError(BioMartError):
    pass


class BioMartQueryError(BioMartError):
    pass


_BaseAttribute = namedtuple(
    "Attribute",
    ["internal_name", "name", "description", "4", "format", "table", "7"],
    rename=True
)


class Attribute(_BaseAttribute):
    """
    An attribute in a BioMart data-set.
    """
    @classmethod
    def from_string(cls, string):
        return cls(string.split("\t"))

    # Re-define the getters for documentation
    internal_name = property(
        itemgetter(0), doc="Attribute's internal name")

    name = property(
        itemgetter(1), doc="Human readable name.")

    description = property(
        itemgetter(2), doc="Human readable description")

    format = property(
        itemgetter(4), doc="Attribute format")

    # For back compatibility.
    internalName = internal_name


_BaseFilter = namedtuple(
    "Filter",
    ["internalName", "name", "values", "description", "type",
     "qualifiers", "6", "attribute_table"],
    rename=True
)


class Filter(_BaseFilter):
    """
    A filter on a BioMart data-set.
    """
    @classmethod
    def from_string(cls, string):
        return cls(string.split("\t"))

    internal_name = property(
        itemgetter(0), doc="Internal name")

    name = property(
        itemgetter(1), doc="Filter name")

    values = property(
        itemgetter(2), doc="List of possible filter values")

    description = property(
        itemgetter(3), doc="Filter description")


class XMLNode(object):

    def __init__(self, tag, attributes, children=[], data=""):
        self.tag = tag
        self.attributes = attributes
        self.children = children if children else []
        self.data = data if data else ""

    @classmethod
    def fromDOMNode(cls, node):
        return XMLNode(node.tagName, dict(node.attributes.items()))

    def _match(self, tag=None, **kwargs):
        match = True
        if tag is not None and self.tag != tag:
            match = False

        return match and all(self.attributes.get(key, None) == value
                             for key, value in kwargs.iteritems())

    def elements(self, tag=None, **kwargs):
        if self._match(tag, **kwargs):
            yield self

        for child in self.children:
            for el in child.elements(tag, **kwargs):
                yield el

    def elements_top(self, tag=None, **kwargs):
        """ Return only the top elements (do not return matching children
        of an already matching element)."
        """
        if self._match(tag, **kwargs):
            yield self
        else:
            for child in self.children:
                for el in child.elements_top(tag, **kwargs):
                    yield el

    def subelements(self, *args, **kwargs):
        return itertools.chain(*[c.elements(*args, **kwargs)
                                 for c in self.children])

    def subelements_top(self, *args, **kwargs):
        return itertools.chain(*[c.elements_top(*args, **kwargs)
                                 for c in self.children])

    def __iter__(self):
        """ Iterate over all sub elements.
        """
        return itertools.chain(*[iter(c) for c in self.children])

    def __repr__(self):
        return 'XMLNode("%s", %s)' % (self.tag, repr(self.attributes))


def parseXML(stream, parser=None):
    if isinstance(stream, basestring):
        events = pulldom.parseString(stream, parser)
    else:
        events = pulldom.parse(stream, parser)

    document = None
    chain = []
    for event, node in events:
        if event == "START_DOCUMENT":
            chain.append(XMLNode("DOCUMENT", {}))

        elif event == "START_ELEMENT":
            node = XMLNode.fromDOMNode(node)
            if chain:
                chain[-1].children.append(node)
            chain.append(node)

        elif event == "END_ELEMENT":
            chain.pop(-1)

        elif event == "CHARACTERS":
            chain[-1].data += node.data

        elif event == "END_DOCUMENT":
            document = chain.pop(-1)
    return document or chain[0]


def de_tab(text, sep="\t"):
    return [line.split(sep) for line in text.splitlines() if line.strip()]


def cached(func):
    @wraps(func)
    def f(self):
        if getattr(self, "_cache_" + func.__name__, None) is None:
            setattr(self, "_cache_" + func.__name__, func(self))
        return getattr(self, "_cache_" + func.__name__)
    return f


def drop_iter(generator):
    """Drop all elements from the iterator that raise an exception.
    """
    def _iter(generator):
        while True:
            try:
                yield generator.next()
            except StopIteration:
                raise StopIteration
            except Exception, ex:
                warnings.warn("An error occured during iteration:\n%s" %
                              str(ex), UserWarning, stacklevel=2)
    return list(_iter(generator))


DEFAULT_ADDRESS = "http://www.biomart.org/biomart/martservice"

DATA_CACHE = os.path.join(environ.buffer_dir, "biomart-data.cache.db")
META_CACHE = os.path.join(environ.buffer_dir, "biomart-mata.cache.db")


def checkBioMartServerError(response):
    if response.strip().startswith("Mart name conflict"):
        raise DatabaseNameConflictError(response)
    elif response.strip().startswith("Problem retrieving datasets"):
        raise BioMartError(response)
    elif response.startswith("non-BioMart die():"):
        raise BioMartServerError(response)


class BioMartConnection(object):

    """ A connection to a BioMart martservice server.

    >>> connection = BioMartConnection(
    ...     "http://www.biomart.org/biomart/martservice")
    >>> response = connection.registry()
    >>> response = connection.datasets(mart="ensembl")

    """
    FOLLOW_REDIRECTS = False

    def __init__(self, address=None):

        self.address = address if address is not None else DEFAULT_ADDRESS

        self._error_cache = {}

    def _get_cache_for_args(self, kwargs):
        if "type" in kwargs:
            return self._open_meta_cache
        elif "query" in kwargs:
            return self._open_data_cache
        else:
            raise ValueError

    def _open_data_cache(self, flag="r"):
        return self._open_cache(DATA_CACHE, flag=flag)

    def _open_meta_cache(self, flag="r"):
        return self._open_cache(META_CACHE, flag=flag)

    def _open_cache(self, filename, flag="r"):
        # Ensure the cache db actualy exists (r,w flags do not create it)
        if flag in ["r", "w"]:
            try:
                return shelve.open(filename, flag)
            except anydbm.error:
                shelve.open(filename, "c").close()
                return shelve.open(filename, flag)
        else:
            return shelve.open(filename, flag)

    def request_url(self, **kwargs):
        order = ["type", "dataset", "mart", "virtualSchema", "query"]
        items = sorted(
           kwargs.iteritems(),
           key=lambda item: order.index(item[0]) if item[0] in order else 10
        )
        query = "&".join("%s=%s" % (key, urllib2.quote(value))
                         for key, value in items if key != "POST")

        return self.address + "?" + query
#         return url.replace(" ", "%20")

    def request(self, **kwargs):
        url = self.request_url(**kwargs)
        cache_key = url
        if isinstance(url, unicode):
            cache_key = url.encode("utf-8")

        if cache_key in self._error_cache:
            raise self._error_cache[cache_key]

        cache_open = self._get_cache_for_args(kwargs)

        with closing(cache_open(flag="r")) as cache:
            response_cached = cache_key in cache
            response = cache[cache_key] if response_cached else None

        if not response_cached:
            try:
                response = urllib2.urlopen(url).read()
                checkBioMartServerError(response)
            except Exception, ex:
                self._error_cache[url] = ex
                raise ex

            with closing(cache_open(flag="w")) as cache:
                cache[cache_key] = response

        return StringIO(response)

    def registry(self, **kwargs):
        return self.request(type="registry")

    def datasets(self, mart="ensembl", **kwargs):
        return self.request(type="datasets", mart=mart, **kwargs)

    def attributes(self, dataset="oanatinus_gene_ensembl", **kwargs):
        return self.request(type="attributes", dataset=dataset, **kwargs)

    def filters(self, dataset="oanatinus_gene_ensembl", **kwargs):
        return self.request(type="filters", dataset=dataset, **kwargs)

    def configuration(self, dataset="oanatinus_gene_ensembl", **kwargs):
        return self.request(type="configuration", dataset=dataset, **kwargs)

    def clear_cache(self):
        with closing(self._open_data_cache(flag="n")):
            pass

        with closing(self._open_meta_cache(flag="n")):
            pass

        self._error_cache.clear()

    # Back compatibility
    clearCache = clear_cache


class BioMartRegistry(object):

    """ A class representing a BioMart registry. Arguments:

    :param stream: A file like object with xml registry or a
          BioMartConnection instance

    >>> registry = tioMartRegistry(connection)
    >>> for schema in registry.virtual_schemas():
    ...    print schema.name
    ...
    default

    """

    def __init__(self, stream):
        self.connection = stream if isinstance(stream, BioMartConnection) \
                          else None
        if self.connection:
            if hasattr(self.connection, "_registry"):
                self.__dict__ = self.connection._registry.__dict__
                return
            else:
                stream = self.connection.registry()
                self.connection._registry = self
        else:
            stream = open(stream, "rb") if isinstance(stream, basestring) \
                     else stream
        self.registry = self.parse(stream)

    @cached
    def virtual_schemas(self):
        """Return a list of :obj:`BioMartVirtualSchema` instances representing
        each schema.
        """
        schemas = [schema.attributes.get(name, "default")
                   for name in self.registry.elements("virtualSchema")]
        if not schemas:
            schemas = [BioMartVirtualSchema(self.registry, name="default",
                                            connection=self.connection)]
        else:
            schemas = [BioMartVirtualSchema(
                            schema,
                            name=schema.attributes.get("name", "default"),
                            connection=self.connection)
                       for schema in self.registry.elements("virtualSchema")]
        return schemas

    def virtual_schema(self, name):
        """ Return a named virtual schema.
        """
        for schema in self.virtual_schemas():
            if schema.name == name:
                return schema
        raise ValueError("Unknown schema name '%s'" % name)

    @cached
    def marts(self):
        """Return a list off all 'mart' instances (:obj:`BioMartDatabase`)
        regardless of their virtual schemas.
        """
        return reduce(list.__add__,
                      drop_iter(schema.marts()
                                for schema in self.virtual_schemas()),
                      [])

    def mart(self, name):
        """ Return a named mart.
        """
        for mart in self.marts():
            if mart.name == name:
                return mart
        raise ValueError("Unknown mart name '%s'" % name)

    def databases(self):
        """Same as :obj:`marts`.
        """
        return self.marts()

    def datasets(self):
        """Return a list of all datasets (:obj:`BioMartDataset`) from all
        marts regardless of their virtual schemas.
        """
        return reduce(list.__add__,
                      drop_iter(mart.datasets()
                                for mart in self.marts()),
                      [])

    def dataset(self, internalName, virtualSchema=None):
        """ Return a BioMartDataset instance that matches the `internalName`.
        """
        if virtualSchema is not None:
            schema = self.virtual_schema(virtualSchema)
            return schema.dataset(internalName)
        else:
            for d in self.datasets():
                if d.internalName == internalName:
                    return d
            raise ValueError("Unknown dataset name: '%s'" % internalName)

    def query(self, **kwargs):
        """Return an initialized :obj:`BioMartQuery` with registry set to self.
        Pass additional arguments to BioMartQuery.__init__ with keyword
        arguments.
        """
        return BioMartQuery(self, **kwargs)

    def links_between(self, exporting, importing, virtualSchema="default"):
        """ Return all links between `exporting` and `importing` datasets in the
        `virtualSchema`.
        """
        schema = [schema for schema in self.virtual_schemas()
                  if schema.name == virtualSchema][0]
        return schema.links_between(exporting, importing)

    def get_path(self, exporting, importing):
        raise NotImplementedError

    def __iter__(self):
        return iter(self.marts())

    def _find_mart(self, name):
        try:
            return [mart for mart in self.marts()
                    if name in [getattr(mart, "name", None),
                                getattr(mart, "internalName", None)]][0]
        except IndexError, ex:
            raise ValueError(name)

    def __getitem__(self, name):
        return self._find_mart(name)

    def search(self, string, relevance=False):
        results = []
        for mart in self.marts():
            results.extend([(rel, mart, dataset)
                            for rel, dataset in mart.search(string, True)])
        results = sorted(results, reverse=True)
        if not relevance:
            results = [(mart, dataset) for _, mart, dataset in results]
        return results

    @classmethod
    def parse(cls, stream, parser=None):
        """Parse the registry file like object and return a DOM like
        description (:obj:`XMLNode`).
        """
        xml = stream.read()
        doc = parseXML(xml, parser)
        return doc

parseRegistry = BioMartRegistry.parse


class BioMartVirtualSchema(object):
    """ A virtual schema.
    """

    def __init__(self, locations=None, name="default", connection=None):
        self.locations = locations
        self.name = name
        self.connection = connection

    @cached
    def marts(self):
        """Return a list off all 'mart' instances (:obj:`BioMartDatabase`)
        in this schema.
        """
        return drop_iter(
            BioMartDatabase(connection=self.connection,
                            **dict((str(key), val)
                                   for key, val in loc.attributes.items()))
            for loc in self.locations.elements("MartURLLocation")
        )

    def databases(self):
        """Same as :obj:`marts`.
        """
        return self.marts()

    @cached
    def datasets(self):
        """Return a list of all datasets (:obj:`BioMartDataset`)
        from all marts in this schema.
        """
        return reduce(list.__add__,
                      drop_iter(mart.datasets()  for mart in self.marts()),
                      [])

    def dataset(self, internalName):
        """Return a dataset with matching `internalName`.
        """
        for dataset in self.datasets():
            if dataset.internalName == internalName:
                return dataset
        raise ValueError("Unknown data set name '%s'" % internalName)

    def links_between(self, exporting, importing):
        """Return a list of link names from `exporting` dataset to
        `importing` dataset.
        """
        exporting = self[exporting]
        importing = self[importing]
        exporting = exporting.configuration().exportables()
        importing = importing.configuration().importables()
        exporting = set([(ex.linkName, getattr(ex, "linkVersion", ""))
                         for ex in exporting])
        importing = set([(im.linkName, getattr(ex, "linkVersion", ""))
                         for im in importing])
        links = exporting.intersection(importing)
        return [link[0] for link in links]

    def links(self):
        """Return a list of (linkName, linkVersion) tuples defined by
        datasets in the schema.
        """
        pass

    def query(self, **kwargs):
        """ Return an initialized :obj:`BioMartQuery` with registry
        and `virtualSchema` set to self. Pass additional arguments to
        :obj:`BioMartQuery.__init__` with keyword arguments

        """
        return BioMartQuery(self, virtualSchema=self.name, **kwargs)

    def __iter__(self):
        return iter(self.marts())

    def __getitem__(self, key):
        try:
            return self.dataset(key)
        except ValueError:
            raise KeyError(key)


class BioMartDatabase(object):
    """A BioMart 'mart' instance.

    :param str name: Name of the mart instance.
    :param str virtualSchema: Name of the virtualSchema of the dataset.
    :param BioMartConnection: An optional BioMartConnection instance.

    """
    host = "www.biomart.org"
    path = "/biomart/martservice"

    def __init__(self, name="ensembl", virtualSchema="default",
                 connection=None, database="ensembl_mart_60", default="1",
                 displayName="ENSEMBL GENES 60 (SANGER UK)",
                 host="www.biomart.org", includeDatasets="", martUser="",
                 path="/biomart/martservice", port="80",
                 serverVirtualSchema="default", visible="1", **kwargs):

        self.name = name
        self.virtualSchema = virtualSchema
        self.database = database,
        self.default = default
        self.displayName = displayName
        self.host = host
        self.includeDatasets = includeDatasets
        self.martUser = martUser
        self.path = path
        self.port = port
        self.serverVirtualSchema = serverVirtualSchema
        self.visible = visible
        self.__dict__.update(kwargs.items())  # why?

        if connection is None:
            connection = BioMartConnection()

        if kwargs.get("redirect", None) == "1" and \
                BioMartConnection.FOLLOW_REDIRECTS:
            redirect = BioMartConnection(
                "http://" + self.host + ":" + self.port + self.path,
                cache=connection.cache)
            try:
                registry = redirect.registry()
                connection = redirect
            except urllib2.HTTPError, ex:
                warnings.warn("'%s' is not responding!, using the default "
                              "original connection. %s" %
                              (redirect.address, str(ex)))

        self.connection = connection

    @cached
    def _datasets_index(self):
        keys = ["datasetType", "internalName", "displayName", "visible",
                "assembly", "_5", "_6", "virtualSchema", "date"]

        try:
            datasets = self.connection.datasets(
                mart=self.name, virtualSchema=self.virtualSchema).read()
        except BioMartError, ex:
            if self.virtualSchema == "default":
                datasets = self.connection.datasets(mart=self.name).read()
            else:
                raise
        datasets = de_tab(datasets)
        return [dict(zip(keys, line)) for line in datasets]

    @cached
    def datasets(self):
        """Return a list of all datasets (:obj:`BioMartDataset`)
        in this database.
        """
        return drop_iter(BioMartDataset(mart=self.name,
                                        connection=self.connection, **dataset)
                         for dataset in self._datasets_index())

    def dataset_attributes(self, dataset, **kwargs):
        """ Return a list of dataset attributes.
        """
        dataset = self._find_dataset(dataset)
        return dataset.attributes()

    def dataset_filters(self, dataset, **kwargs):
        """ Return a list of dataset filters.
        """
        dataset = self._find_dataset(dataset)
        return dataset.filters()

    def dataset_query(self, dataset, filters=[], attributes=[]):
        """ Return an dataset query based on dataset, filters and attributes.
        """
        dataset = self._find_dataset(dataset)
        return BioMartQuery(self.con, [dataset], filters, attributes).run()

    def _find_dataset(self, internalName):
        for dataset in self.datasets():
            if dataset.internalName == internalName:
                return dataset
        raise ValueError("Uknown dataset name '%s'" % internalName)

    def __getitem__(self, name):
        try:
            return self._find_dataset(name)
        except ValueError:
            raise KeyError(name)

    def search(self, string, relevance=False):
        strings = string.lower().split()
        results = []
        for dataset, conf in drop_iter((dataset, dataset.configuration())
                                       for dataset in self.datasets()):
            trees = (conf.attribute_pages() +
                     conf.attribute_groups() +
                     conf.attribute_collections() +
                     conf.attributes())

            names = (" ".join([getattr(tree, "displayName", "")
                               for tree in trees])
                     .lower())
            count = sum([names.count(s) for s in strings])
            if count:
                results.append((count, dataset))
        return results


class BioMartDataset(object):
    """
    A BioMart dataset (returned by :obj:`BioMartDatabase`).
    """

    # these attributes store the result of
    # ".../martservice?type=datasets&mart=..." query
    FIELDS = ["datasetType", "internalName", "displayName", "visible",
              "assembly", "_5", "_6", "virtualSchema", "date"]

    def __init__(self, mart="ensembl", internalName="hsapiens_gene_ensembl",
                 virtualSchema="default", connection=None,
                 datasetType="TableSet", displayName="", visible="1",
                 assembly="", date="", **kwargs):
        self.connection = \
            connection if connection is not None else BioMartConnection()
        self.mart = mart
        self.internalName = internalName
        self.virtualSchema = virtualSchema
        self.datasetType = datasetType
        self.displayName = displayName
        self.visible = visible
        self.assembly = assembly
        self.date = date
        self.__dict__.update(kwargs)
        self._attributes = None
        self._filters = None

    @cached
    def attributes(self):
        """Return a list of available attributes for this dataset
        (:obj:`Attribute`).
        """
        stream = self.connection.attributes(
            dataset=self.internalName, virtualSchema=self.virtualSchema
        )
        response = stream.read()
        lines = response.splitlines()
        return [Attribute.from_string(line) for line in lines if line.strip()]

    @cached
    def filters(self):
        """Return a list of available filters for this dataset
        (:obj:`Filter`).
        """
        stream = self.connection.filters(
            dataset=self.internalName, virtualSchema=self.virtualSchema
        )
        response = stream.read()
        lines = response.splitlines()
        return [Filter.from_string(line) for line in lines if line.strip()]

    @cached
    def configuration(self, parser=None):
        """Return the configuration tree for this dataset
        (:obj:`DatasetConfig`).
        """
        stream = self.connection.configuration(
            dataset=self.internalName, virtualSchema=self.virtualSchema
        )
        response = stream.read()
        doc = parseXML(response, parser)
        config = list(doc.elements("DatasetConfig"))[0]
        return DatasetConfig(BioMartRegistry(self.connection), config.tag,
                             config.attributes, config.children)

    def get_data(self, attributes=[], filters=[], unique=False):
        """Construct and run a :obj:`BioMartQuery` and return its results.
        """
        query = BioMartQuery(self.connection, dataset=self,
                             attributes=attributes, filters=filters,
                             uniqueRows=unique,
                             virtualSchema=self.virtualSchema)
        return query.run()

    def count(self, filters=[], unique=False):
        """Construct and run a :obj:`BioMartQuery` and count the number of returned
        lines.
        """
        query = BioMartQuery(self.connection, dataset=self, filters=filters,
                             uniqueRows=unique,
                             virtualSchema=self.virtualSchema)
        return query.get_count()

    def get_example_table(self, attributes=[], filters=[], unique=False):
        query = BioMartQuery(self.connection, dataset=self,
                             attributes=attributes, filters=filters,
                             uniqueRows=unique,
                             virtualSchema=self.virtualSchema)
        return query.get_example_table()


class BioMartQuery(object):
    """ Construct a query to run on a BioMart server.

    >>> BioMartQuery(connection,
    ...              dataset="hsapiens_gene_ensembl",
    ...              attributes=["ensembl_transcript_id",
    ...                          "chromosome_name"],
    ...              filters=[("chromosome_name", ["22"])]).get_count()
    1221
    >>> # Equivalent to
    >>> query = BioMartQuery(connection)
    >>> query.set_dataset("hsapiens_gene_ensembl")
    >>> query.add_filter("chromosome_name", "22")
    >>> query.add_attribute("ensembl_transcript_id")
    >>> query.add_attribute("chromosome_name")
    >>> query.get_count()
    1221

    """
    class XMLQuery(object):
        XML = """<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE Query>
<Query virtualSchemaName = "%(virtualSchemaName)s" formatter = "%(formatter)s" header = "%(header)s" uniqueRows = "%(uniqueRows)s" count = "%(count)s" datasetConfigVersion = "%(version)s" >  
%(datasets_xml)s
</Query>"""
        version = "0.6"

        def __init__(self, query):
            self.query = query
            self._query = query._query
            self.__dict__.update(query.__dict__)

        def get_xml(self, count=None, header=False):
            datasets_xml = "\n".join(self.query_dataset(*query)
                                     for query in self._query)

            links_xml = self.query_links(self._query)
            datasets_xml += "\n" + links_xml

            count = self.count if count is None else count
            args = dict(datasets_xml=datasets_xml,
                        uniqueRows="1" if self.uniqueRows else "0",
                        count="1" if count else "",
                        header="1" if header else "0",
                        virtualSchemaName=self.virtualSchema,
                        formatter=self.format,
                        version=self.version)
            xml = self.XML % args
            return xml

        def query_attributes(self, attributes):
            def name(attr):
                if isinstance(attr, Attribute):
                    return attr.internalName
                elif isinstance(attr, AttributeDescription):
                    return attr.internalName
                else:
                    return str(attr)

            xml = "\n".join('\t\t<Attribute name = "%s" />' %
                            name(attr).lower()
                            for attr in attributes)
            return xml

        def query_filter(self, filters):
            def name(filter):
                if isinstance(filter, Filter):
                    return filter.internalName
                elif isinstance(filter, FilterDescription):
                    return getattr(filter, "field",
                                   getattr(filter, "internalName"))
                else:
                    return str(filter)

            def args(args):
                if isinstance(args, list):
                    return 'value="%s"' % ",".join(args)
                elif isinstance(args, basestring):
                    return 'value="%s"' % args
                elif isinstance(args, dict):
                    return " ".join(['%s="%s"' % (key, value)
                                     for key, value in args.iteritems()])

            xml = "\n".join(
                '\t\t<Filter name = "%s" %s />' % (name(fil), args(val))
                for fil, val in filters
            )
            return xml

        def query_dataset(self, dataset, attributes, filters):
            name, interface = (dataset.internalName, "default") if \
                isinstance(dataset, BioMartDataset) else (dataset, "default")
            xml = ('\t<Dataset name = "%s" interface = "%s" >\n' %
                   (name, interface))
            xml += self.query_attributes(attributes) + "\n"
            xml += self.query_filter(filters)
            xml += '\n\t</Dataset>'
            return xml

        def query_links(self, query):
            def name(dataset):
                if isinstance(dataset, BioMartDataset):
                    return getattr(dataset, "internalName",
                                   getattr(dataset, "name", None))
                else:
                    return str(dataset)

            if len(query) == 2:
                return ('\t<Links source="%s" target="%s"/>' %
                        (name(query[0][0]), name(query[1][0])))
            else:
                return ""

    class XMLQueryOld(XMLQuery):
        version = "0.4"

        def query_filter(self, filters):
            def name(filter):
                if isinstance(filter, Filter):
                    return filter.internalName
                elif isinstance(filter, FilterDescription):
                    return getattr(filter, "field",
                                   getattr(filter, "internalName"))
                else:
                    return str(filter)

            def value(value):
                return (str(value) if not isinstance(value, list)
                        else ",".join([str(v) for v in value]))

            xml = "\n".join(
                ('\t\t<ValueFilter name = "%s" value="%s" />' %
                 (name(fil), value(val)))
                for fil, val in filters)
            return xml

    def __init__(self, registry, virtualSchema="default", dataset=None,
                 attributes=[], filters=[], count=False, uniqueRows=False,
                 format="TSV"):
        if isinstance(registry, BioMartConnection):
            self.registry = BioMartRegistry(registry)
            self.virtualSchema = virtualSchema
        elif isinstance(registry, BioMartVirtualSchema):
            self.registry = registry.connection
            self.virtualSchema = registry.name
        else:
            self.registry = registry
            self.virtualSchema = virtualSchema

        self._query = []
        if dataset:
            self.set_dataset(dataset)
            for attr in attributes:
                self.add_attribute(attr)
            for filter, value in filters:
                self.add_filter(filter, value)
        self.count = count
        self.uniqueRows = uniqueRows
        self.format = format

    def set_dataset(self, dataset):
        self._query.append((dataset, [], []))

    def add_filter(self, filter, value):
        self._query[-1][2].append((filter, value))

    def add_attribute(self, attribute):
        self._query[-1][1].append(attribute)

    def get_count(self):
        count = self.run(count=True)
        if count.strip():
            count = int(count.strip())
        else:
            count = 0
        return count

    def run(self, count=None, header=False):
        query = (self.xml_query(count=count, header=header)
                     .replace("\n", "").replace("\t", ""))
        stream = self.registry.connection.request(query=query)
        stream = stream.read()
        if stream.startswith("Query ERROR:"):
            raise BioMartQueryError(stream)
        return stream

    def set_unique(self, unique=False):
        self.uniqueRows = unique

    def xml_query(self, count=None, header=False):
        self.version = version = "0.4"

        schema = self.virtualSchema

        dataset = self._query[0][0]
        dataset = dataset.internalName if isinstance(dataset, BioMartDataset) \
                  else dataset

        dataset = self.registry.dataset(
            dataset, virtualSchema=self.virtualSchema)

        conf = dataset.configuration()
        self.version = version = getattr(conf, "softwareVersion", "0.4")

        if version > "0.4":
            xml = self.XMLQuery(self).get_xml(count, header)
        else:
            xml = self.XMLQueryOld(self).get_xml(count, header)
        return xml

    def get_example_table(self):
        import orange
        data = self.run(count=False, header=True)

        if self.format.lower() == "tsv":
            header, data = data.split("\n", 1)
            domain = orange.Domain([orange.StringVariable(name)
                                    for name in header.split("\t")], False)
            data = [line.split("\t")
                    for line in data.split("\n") if line.strip()]
            return orange.ExampleTable(domain, data) if data else None
        elif self.format.lower() == "fasta":
            domain = orange.Domain(
                [orange.StringVariable("id"),
                 orange.StringVariable("sequence")],
                False)  # TODO: meaningful id
            examples = []
            from StringIO import StringIO
            from Bio import SeqIO
            for seq in SeqIO.parse(StringIO(data), "fasta"):
                examples.append([seq.id, str(seq.seq)])
            return orange.ExampleTable(domain, examples)
        else:
            raise BioMartError("Unsupported format: %" % self.format)


def get_pointed(self):
    if self.is_pointer():
        pointerDataset = self.pointerDataset
        if hasattr(self, "pointerAttribute"):
            name, getter = self.pointerAttribute, "AttributeDescription"
        elif hasattr(self, "pointerFilter"):
            name, getter = self.pointerFilter, "FilterDescription"

        dataset = self.registry.dataset(pointerDataset)

        conf = dataset.configuration()
        desc = list(conf.elements(getter, internalName=name))
        if desc:
            return dataset, desc[0]
        else:
            warnings.warn("Could not resolve pointer '%s' in '%s'" %
                          (name, pointerDataset), UserWarning, stacklevel=2)
            return None, None


def is_pointer(self):
    return hasattr(self, "pointerAttribute") or hasattr(self, "pointerFilter")


class ConfigurationNode(XMLNode):

    def __init__(self, registry, *args, **kwargs):
        XMLNode.__init__(self, *args, **kwargs)
        self.registry = registry
        self.__dict__.update([(self._name_mangle(name), value) \
                              for name, value in self.attributes.iteritems()])

        self.children = [self._factory(child) for child in self.children]

    def _name_mangle(self, name):
        if name in self.__dict__:
            return name + "_"
        else:
            return name

    def _factory(self, node):
        tag = node.tag
        class_ = globals().get(tag, None)
        if not class_:
            raise ValueError("Unknown node '%s;" % tag)
        else:
            return class_(self.registry, tag, node.attributes, node.children)

    def elements(self, tag=None, **kwargs):
        if isinstance(tag, type):
            tag = tag.__name__
        return XMLNode.elements(self, tag, **kwargs)

    def elements_top(self, tag=None, **kwargs):
        if isinstance(tag, type):
            tag = tag.__name__
        return XMLNode.elements_top(self, tag, **kwargs)

    def __repr__(self):
        return ('%s("%s", %s)' %
                (self.__class__.__name__, self.tag, repr(self.attributes)))


class DatasetConfig(ConfigurationNode):
    pass


class Exportable(ConfigurationNode):
    pass


class Importable(ConfigurationNode):
    pass


class FilterPage(ConfigurationNode):
    pass


class FilterGroup(ConfigurationNode):
    pass


class FilterCollection(ConfigurationNode):
    pass


class FilterDescription(ConfigurationNode):

    is_pointer = is_pointer
    get_pointed = get_pointed


class AttributePage(ConfigurationNode):
    pass


class AttributeGroup(ConfigurationNode):
    pass


class AttributeCollection(ConfigurationNode):
    pass


class AttributeDescription(ConfigurationNode):

    is_pointer = is_pointer
    get_pointed = get_pointed


class Option(ConfigurationNode):
    pass


class PushAction(ConfigurationNode):
    pass


class MainTable(ConfigurationNode):
    pass


class Key(ConfigurationNode):
    pass


if __name__ == "__main__":
    con = BioMartConnection("http://www.biomart.org/biomart/martservice")
    registry = BioMartRegistry(con)
    for schema in registry.virtual_schemas():
        print "Virtual schema '%s'" % schema.name
        for mart in schema.databases():
            print "\tMart: '%s' ('%s'):" % (mart.name, mart.displayName)
            for dataset in mart.datasets():
                print "\t\t Dataset '%s' %s' '%s'" % \
                      (dataset.datasetType, dataset.internalName,
                       dataset.displayName)

    database = BioMartDatabase(name="dicty", connection=con)
    datasets = database.datasets()
    print datasets
    dataset = datasets[2]
    configuration = dataset.configuration()
    attr = dataset.attributes()
    print attr
    filters = dataset.filters()
    print filters
    reg = BioMartRegistry(con)

    dataset = reg.dataset("scerevisiae_gene_ensembl")
    query = BioMartQuery(
         con, dataset="scerevisiae_gene_ensembl",
         attributes=["ensembl_transcript_id", "transcript_exon_intron"],
         filters=[("chromosome_name", "I"),
                  ("with_wikigene", {"excluded": "1"})],
         format="FASTA")
    print query.xml_query()
    print query.run()

    data = query.get_example_table()
    data.save("seq.tab")

    import doctest
    doctest.testmod(extraglobs={"connection": con, "registry": registry},
                    optionflags=doctest.ELLIPSIS)



from __future__ import absolute_import, division

from collections import defaultdict
import cPickle, os, shutil, sys, StringIO, tarfile, urllib2


from Orange.orng import orngEnviron, orngServerFiles

_COMMON_NAMES = (
    ("3702",   "Arabidopsis thaliana"),
    ("9913",   "Bos taurus"),
    ("6239",   "Caenorhabditis elegans"),
    ("3055",   "Chlamydomonas reinhardtii"),
    ("7955",   "Danio rerio"),
    ("352472", "Dictyostelium discoideum AX4"),
    ("7227",   "Drosophila melanogaster"),
    ("562",    "Escherichia coli"),
    ("11103",  "Hepatitis C virus"),
    ("9606",   "Homo sapiens"),
    ("10090",  "Mus musculus"),
    ("2104",   "Mycoplasma pneumoniae"),
    ("4530",   "Oryza sativa"),
    ("5833",   "Plasmodium falciparum"),
    ("4754",   "Pneumocystis carinii"),
    ("10116",  "Rattus norvegicus"),
    ("4932",   "Saccharomyces cerevisiae"),
    ("4896",   "Schizosaccharomyces pombe"),
    ("31033",  "Takifugu rubripes"),
    ("8355",   "Xenopus laevis"),
    ("4577",   "Zea mays")
)

_COMMON_NAMES_MAPPING = dict(_COMMON_NAMES)


# list of common organisms from http://www.ncbi.nlm.nih.gov/Taxonomy
def common_taxids():
    """Return taxonomy IDs for common organisms."""
    # Sorted lexicographically by names
    return [taxid for taxid, _ in _COMMON_NAMES]


def common_taxid_to_name(taxid):
    """Return a name for a common organism taxonomy id."""
    return _COMMON_NAMES_MAPPING[taxid]


def taxname_to_taxid(name):
    """Return taxonomy ID for a taxonomy name."""
    name_to_taxid = dict(map(reversed, _COMMON_NAMES))

    if name in name_to_taxid:
        return name_to_taxid[name]
    return None


def essential_taxids():
    """Return taxonomy IDs for organisms that are included in (default)
    Orange Bioinformatics installation.
    """
    return ["352472", # Dictyostelium discoideum
            "7227",  # Drosophila melanogaster
            "9606",  # Homo sapiens
            "10090", # Mus musculus
            "4932",  # Saccharomyces cerevisiae
            ]


def shortname(taxid):
    """ Short names for common_taxids organisms """
    names = {
    "3702"  : ["arabidopsis", "thaliana", "plant"],
    "9913"  : ["cattle", "cow"],
    "6239"  : ["nematode", "roundworm"],
    "3055"  : ["algae"],
    "7955"  : ["zebrafish"],
    "352472": ["dicty", "amoeba", "slime mold"],
    "7227"  : ["fly", "fruit fly", "vinegar fly"],
    "562"   : ["ecoli", "coli", "bacterium"],
    "11103" : ["virus, hepatitis"],
    "9606"  : ["human"],
    "10090" : ["mouse", "mus"],
    "2104"  : ["bacterium", "mycoplasma"],
    "4530"  : ["asian rice", "rice", "cereal", "plant"],
    "5833"  : ["plasmodium", "malaria", "parasite"],
    "4754"  : ["pneumonia", "fungus"],
    "10116" : ["rat", "laboratory rat"],
    "4932"  : ["yeast", "baker yeast", "brewer yeast"],
    "4896"  : ["yeast", "fission yeast"],
    "31033" : ["fish", "pufferfish"],
    "8355"  : ["frog", "african clawed frog"],
    "4577"  : ["corn", "cereal grain", "plant"]
    }
    if taxid in names.keys():
        return names[taxid]
    return []

default_database_path = os.path.join(orngEnviron.bufferDir, "bigfiles", "Taxonomy")

class MultipleSpeciesException(Exception):
    pass

class UnknownSpeciesIdentifier(Exception):
    pass

def pickled_cache(filename=None, dependencies=[], version=1, maxSize=30):
    """ Return a cache function decorator. 
    """
    def datetime_info(domain, filename):
        try:
            return orngServerFiles.info(domain, filename)["datetime"]
        except IOError:
            return orngServerFiles.ServerFiles().info(domain, filename)["datetime"]
            
    def cached(func):
        default_filename = os.path.join(orngEnviron.bufferDir, func.__module__ + "_" + func.__name__ + "_cache.pickle")
        def f(*args, **kwargs):
            currentVersion = tuple([datetime_info(domain, file) for domain, file in dependencies]) + (version,)
            try:
                cachedVersion, cache = cPickle.load(open(filename or default_filename, "rb"))
                if cachedVersion != currentVersion:
                    cache = {}
            except IOError, er:
                cacheVersion, cache = "no version", {}
            allArgs = args + tuple([(key, tuple(value) if type(value) in [set, list] else value)\
                                     for key, value in kwargs.items()])
            if allArgs in cache:
                return cache[allArgs]
            else:
                res = func(*args, **kwargs)
                if len(cache) > maxSize:
                    del cache[iter(cache).next()]
                cache[allArgs] = res
                cPickle.dump((currentVersion, cache), open(filename or default_filename, "wb"), protocol=cPickle.HIGHEST_PROTOCOL)
                return res
        return f

    return cached

def cached(func):
    """Cached one arg method
    """
    def f(self, arg):
        if arg not in self._cache:
            self._cache[arg] = func(self, arg)
        return self._cache[arg]
    f._cache = {}
    f.__name__ = "Cached " + func.__name__
    return f
    
class TextDB(object):
    entry_start_string = chr(255)
    entry_end_string = chr(254)+"\n"
    entry_separator_string = chr(253)
    
    @property
    def _text_lower(self):
        if id(self._text) == self._lower_text_id:
            return self._lower_text
        else:
            self._lower_text_id = id(self._text)
            self._lower_text = self._text.lower()
            return self._lower_text
        
    def __init__(self, file=None, **kwargs):
        self._text = ""
        self._lower_text_id = id(self._text) - 1
        self._cache = {}
        self.__dict__.update(kwargs)
        
        if file != None:
            self._text = open(file, "rb").read()

    def _find_all(self, string, start=0, text=None, unique=True):
        text = text if text != None else self._text_lower
        while True:
            index = text.find(string, start)
            if index != -1:
                yield index
                if unique:
                    start = text.find(self.entry_start_string, index + 1)
                else:
                    start = index + 1
            else:
                raise StopIteration

    def _get_entry_at(self, index, text=None):
        text = text if text != None else self._text
        start = text.rfind(self.entry_start_string, 0, index + 1)
        end = text.find(self.entry_end_string, index)
        return self._text[start+1:end]

    @cached
    def get_entry(self, id):
        try:
            index = self._find_all(self.entry_start_string + id + self.entry_separator_string).next()
        except StopIteration:
            raise KeyError, id
        return self._get_entry_at(index)
                
    def search(self, string):
        string = string.lower()
        res = []
        for idx in self._find_all(string):
            entry = self._get_entry_at(idx)
            id , rest = entry.split(self.entry_separator_string, 1)
            self._cache[id] = entry
            res.append(id)
        return res

    def insert(self, entry):
        self._text += self.entry_start_string + self.entry_separator_string.join(entry) + self.entry_end_string

    def __iter__(self):
        for idx in self._find_all(self.entry_start_string):
            entry = self._get_entry_at(idx)
            if entry:
                yield entry.split(self.entry_separator_string ,1)[0]

    def __getitem__(self, id):
        entry = self.get_entry(id)
        return entry.split(self.entry_separator_string)[1:]

    def __setitem__(self, id, entry):
        self.insert([id] + list(entry))

    def create(self, filename):
        f = open(filename, "wb")
        f.write(self._text)
        def write(entry):
            f.write(self.entry_start_string + self.entry_separator_string.join(entry) + self.entry_end_string)
        return write
    
class Taxonomy(object):
    __shared_state = {"_text":None, "_info":None}
    def __init__(self):
        self.__dict__ = self.__shared_state
        if not self._text:
            self.Load()
            
    def Load(self):
        try:
            self._text = TextDB(os.path.join(default_database_path, "ncbi_taxonomy.tar.gz", "ncbi_taxonomy.db"))
            self._info = TextDB(os.path.join(default_database_path, "ncbi_taxonomy.tar.gz", "ncbi_taxonomy_inf.db"))
            return
        except Exception, ex:
            pass
        try:
            orngServerFiles.download("Taxonomy", "ncbi_taxonomy.tar.gz")
            self._text = TextDB(os.path.join(default_database_path, "ncbi_taxonomy.tar.gz", "ncbi_taxonomy.db"))
            self._info = TextDB(os.path.join(default_database_path, "ncbi_taxonomy.tar.gz", "ncbi_taxonomy_inf.db"))
            return
        except Exception, ex:
            raise

    def get_entry(self, id):
        try:
            entry = self._text[id]
        except KeyError:
            raise UnknownSpeciesIdentifier
        return entry
                
    def search(self, string, onlySpecies=True):
        res = self._text.search(string)
        if onlySpecies:
            res = [r for r in res if "species" in self._text[r][1]]
        return res

    def __iter__(self):
        return iter(self._text)

    def __getitem__(self, id):
        entry = self.get_entry(id)
        return entry[2] ## item with index 2 is allways scientific name

    def other_names(self, id):
        entry = self.get_entry(id)
        info = self._info[id]
        names = entry[2:] ## index 2 and larger are names
        return list(zip(names, info))[1:] ## exclude scientific name

    def rank(self, id):
        entry = self.get_entry(id)
        return entry[1]

    def parent(self, id):
        entry = self.get_entry(id)
        return entry[0]

    def subnodes(self, id, levels=1):
        res = self._text.search(self._text.entry_separator_string + id + self._text.entry_separator_string)
        res = [r for r in res if self.get_entry(r)[0] == id]
        if levels > 1:
            for r in list(res):
                res.extend(self.subnodes(r, levels-1))
        return res

    def taxids(self):
        return list(self)
    
    @staticmethod
    def ParseTaxdumpFile(file=None, outputdir=None, callback=None):
        from cStringIO import StringIO
        import Orange.utils
        if file == None:
            so = StringIO()
            Orange.utils.wget("ftp://ftp.ncbi.nih.gov/pub/taxonomy/taxdump.tar.gz", dst_obj=so)
            file = tarfile.open(None, "r:gz", StringIO(so.getvalue()))
            so.close()
        elif type(file) == str:
            file = tarfile.open(file)
        names = file.extractfile("names.dmp").readlines()
        nodes = file.extractfile("nodes.dmp").readlines()
        namesDict = defaultdict(list)
        for line in names:
            if not line.strip():
                continue
            line = line.rstrip("\t\n|").split("\t|\t")
            id, name, unique_name, name_class = line
            if unique_name:
                namesDict[id].append((unique_name , name_class))
            else:
                namesDict[id].append((name , name_class))

        nodesDict = {}
        for line in nodes:
            if not line.strip():
                continue
            line = line.split("\t|\t")[:3]
            id, parent, rank = line
            nodesDict[id] = (parent, rank)
        
        name_class_codes = defaultdict(iter(range(255)).next)
        name_class_codes["scientific name"] ## Force scientific name to be first
        if outputdir == None:
            outputdir = default_database_path
        text = TextDB().create(os.path.join(outputdir, "ncbi_taxonomy.db"))
        info = TextDB().create(os.path.join(outputdir, "ncbi_taxonomy_inf.db"))
        milestones = set(range(0, len(namesDict), max(int(len(namesDict)/100), 1)))
        for i, (id, names) in enumerate(namesDict.items()):
            parent, rank = nodesDict[id]
            ## id, parent and rank go first
            entry = [id, parent, rank]
            ## all names and name class codes pairs follow ordered so scientific name is first
            names = sorted(names, key=lambda (name, class_): name_class_codes[class_])
            entry.extend([name for name ,class_ in names])
            info_entry = [id] + [class_ for name, class_ in names]
            text(entry)
            info(info_entry)
            if callback and i in milestones:
                callback(i)


@pickled_cache(None, [("Taxonomy", "ncbi_taxonomy.tar.gz")], version=1)
def name(taxid):
    """
    Return the scientific name for organism with taxid.
    """
    # Most of the lookups will be for the common names, so in most
    # situations we can avoid loading the taxonomy.
    if taxid in _COMMON_NAMES:
        return _COMMON_NAMES[taxid]
    else:
        return Taxonomy()[taxid]


@pickled_cache(None, [("Taxonomy", "ncbi_taxonomy.tar.gz")], version=1)
def other_names(taxid):
    """
    Return a list of (name, name_type) tuples excluding the scientific name.

    Use :func:`name` to retrieve the scientific name.

    """
    return  Taxonomy().other_names(taxid)


@pickled_cache(None, [("Taxonomy", "ncbi_taxonomy.tar.gz")], version=1)
def search(string, onlySpecies=True, exact=False):
    """ Search the NCBI taxonomy database for an organism.

    :param string: Search string.
    :param onlySpecies: Return only taxids of species (and subspecies).
    :param exact:  Return only taxids of organism that exactly match the string.
    """
    ids = Taxonomy().search(string, onlySpecies)
    if exact:
        ids = [id for id in ids if string in [name(id)] + [t[0] for t in other_names(id)]]
    return ids

def lineage(taxid):
    """ Return a list of taxids ordered from the topmost node (root) to taxid.
    """
    tax = Taxonomy()
    result = [taxid]
    while True:
        parent = tax.parent(result[-1])
        result.append(parent)
        if tax[parent] == "root" or parent=="1":
            break
    result.reverse()
    return result


def to_taxid(code, mapTo=None):
    """
    See if the code is a valid code in GO or KEGG and return a set of its taxids.
    """
    try:
        name(code)
        results = set([code])
    except UnknownSpeciesIdentifier:
        results = set()
        from . import kegg, go
        for test in [kegg.to_taxid, go.to_taxid]:
            try:
                r = test(code)
                if type(r) == set:
                    results.update(r)
                else:
                    results.add(r)
            except Exception, ex:
                pass

    if mapTo:
        mapped = [[parent_id for parent_id in mapTo if parent_id in lineage(r)].pop() for r in results if any(parent_id in lineage(r) for parent_id in mapTo)]
        if not mapped:
            t = Taxonomy()
            subnodes = dict([(r, t.subnodes(r, 4)) for r in results])
            mapped = [id for id in mapTo if any(id in subnodes[r] for r in results)]
        results = mapped

    return results


def taxids():
    """ Returns a list of all (about half a million!) NCBI's taxonomy ID's. """
    return Taxonomy().taxids()


def ensure_downloaded(callback=None, verbose=True):
    """
    Retrieve the taxonomy database if not already downloaded.
    """
    orngServerFiles.localpath_download("Taxonomy", "ncbi_taxonomy.tar.gz",
                                       callback=callback, verbose=verbose)


if __name__ == "__main__":
    ids = search("Homo sapiens")
    print ids
    print other_names(ids[0])



"""
<name>Data Profiles</name>
<description>Visualization of data profiles (e.g., time series).</description>
<contact>Tomaz Curk</contact>
<icon>icons/ExpressionProfiles.svg</icon>
<priority>1030</priority>
"""

from Orange.OrangeWidgets import OWGUI
from Orange.OrangeWidgets.OWGraph import *
from Orange.OrangeWidgets.OWToolbars import ZoomSelectToolbar
from Orange.OrangeWidgets.OWTools import *
from Orange.OrangeWidgets.OWWidget import *
import statc

NAME = "Data Profiles"
DESCRIPTION = "Visualization of data profiles (e.g., time series)."
ICON = "icons/ExpressionProfiles.svg"
PRIORITY = 1030

INPUTS = [("Examples", Orange.data.Table, "data", Multiple + Default)]
OUTPUTS = [("Examples", Orange.data.Table)]

REPLACES = ["_bioinformatics.widgets.OWDisplayProfiles.OWDisplayProfiles"]

## format of data:
##     largestAdjacentValue
##     yq3
##     yavg
##     yqM
##     yq1
##     smallestAdjacentValue

class boxPlotQwtPlotCurve(QwtPlotCurve):
    def __init__(self, text = None, connectPoints = 1, tickXw = 1.0/5.0):
        QwtPlotCurve.__init__(self, QwtText(text))
        self.connectPoints = connectPoints
        self.tickXw = tickXw
        self.boxPlotPenWidth = 2

    def draw(self, p, xMap, yMap, f=0, t=-1):
        # save ex settings
##        pen = p.pen()
##        brush = p.brush()
        p.save()
        if type(f)==QRect:
            f = 0 
        
        p.setBackgroundMode(Qt.OpaqueMode)
        p.setPen(self.pen())
        if self.style() == QwtPlotCurve.UserCurve:
            back = p.backgroundMode()
            p.setBackgroundMode(Qt.OpaqueMode)
            if t < 0: t = self.dataSize() - 1

            if f % 6 <> 0: f -= f % 6
            if t % 6 <> 0:  t += 6 - (t % 6)
            ## first connect averages
            first = 1
            if self.connectPoints: 
                for i in range(f, t, 6):
#                    py = yqM = yMap.transform(self.y(i + 3))
                    py = yavg = yMap.transform(self.y(i + 2))
                    px = xMap.transform(self.x(i))
                    if first:
                        first = 0
                    else:
                        p.drawLine(ppx, ppy, px, py)
                    ppx = px
                    ppy = py

            ## then draw boxes
            if self.boxPlotPenWidth > 0:
                np = QPen(self.pen())
                np.setWidth(self.boxPlotPenWidth)
                p.setPen(np)
                for i in range(f, t, 6):
                    largestAdjVal = yMap.transform(self.y(i))
                    yq3 = yMap.transform(self.y(i + 1))
                    yavg = yMap.transform(self.y(i + 2))
                    yqM = yMap.transform(self.y(i + 3))
                    yq1 = yMap.transform(self.y(i + 4))
                    smallestAdjVal = yMap.transform(self.y(i + 5))

                    px = xMap.transform(self.x(i))
                    wxl = xMap.transform(self.x(i) - self.tickXw/2.0)
                    wxr = xMap.transform(self.x(i) + self.tickXw/2.0)

                    p.drawLine(wxl, largestAdjVal, wxr,   largestAdjVal) ## - upper whisker
                    p.drawLine(px, largestAdjVal, px, yq3)               ## | connection between upper whisker and q3
                    p.drawRect(wxl, yq3, wxr - wxl, yq1 - yq3)           ## box from q3 to q1
                    p.drawLine(wxl, yqM, wxr, yqM)                       ## median line
                    p.drawLine(px, yq1, px, smallestAdjVal)              ## | connection between q1 and lower whisker
                    p.drawLine(wxl, smallestAdjVal, wxr, smallestAdjVal) ## _ lower whisker

                    ## average line (circle)
                    p.drawEllipse(px - 3, yavg - 3, 6, 6)

            p.setBackgroundMode(back)
        else:
            QwtPlotCurve.draw(self, p, xMap, yMap, f, t)

        # restore ex settings
        p.restore()
##        p.setPen(pen)
##        p.setBrush(brush)

    def setBoxPlotPenWidth(self, v):
        self.boxPlotPenWidth = v
        self.itemChanged()

class profilesGraph(OWGraph):
    def __init__(self, master, parent = None, name = None, title = ""):
        OWGraph.__init__(self, parent, name)
        self.master = master
        self.setYRlabels(None)
        self.enableGridXB(0)
        self.enableGridYL(0)
        self.setAxisMaxMajor(QwtPlot.xBottom, 10)
        self.setAxisMaxMinor(QwtPlot.xBottom, 0)
        self.setAxisMaxMajor(QwtPlot.yLeft, 10)
        self.setAxisMaxMinor(QwtPlot.yLeft, 5)
        self.setShowMainTitle(1)
        self.setMainTitle(title)
        self.setAxisAutoScale(QwtPlot.xBottom)
        self.setAxisAutoScale(QwtPlot.xTop)
        self.setAxisAutoScale(QwtPlot.yLeft)
        self.setAxisAutoScale(QwtPlot.yRight)

        self.showAverageProfile = 1
        self.showSingleProfiles = 0
        self.curveWidth = 1
        self.renderAntialiased = True
##        self.groups = [('grp1', ['0', '2', '4']), ('grp2', ['4', '6', '8', '10', '12', '14']), ('grp3', ['16', '18'])]

        self.selectedCurves = []
        self.highlightedCurve = None
        self.removeCurves()
##        self.connect(self, SIGNAL("plotMouseMoved(const QMouseEvent &)"), self.onMouseMoved)

    def removeCurves(self):
##        OWGraph.removeCurves(self)
        self.clear()
        self.classColor = None
        self.classBrighterColor = None
        self.profileCurveKeys = []
        self.averageProfileCurveKeys = []
        self.showClasses = []

    def setData(self, data, classColor, classBrighterColor, ShowAverageProfile, ShowSingleProfiles, progressBar = None):
        self.removeCurves()
        self.classColor = classColor
        self.classBrighterColor = classBrighterColor
        self.showAverageProfile = ShowAverageProfile
        self.showSingleProfiles = ShowSingleProfiles

        self.groups = [('grp', data.domain.attributes)]
        ## remove any non continuous attributes from list
        ## at the same time convert any attr. string name into orange var type
        filteredGroups = []
        for (grpname, grpattrs) in self.groups:
            filteredGrpAttrs = []
            for a in grpattrs:
                var = data.domain[a]
                if (var.varType == orange.VarTypes.Continuous):
                    filteredGrpAttrs.append(var)
                else:
                    print "warning, skipping attribute:", a
            if len(filteredGrpAttrs) > 0:
                filteredGroups.append( (grpname, filteredGrpAttrs) )
        self.groups = filteredGroups
        
        ## go group by group
        avgCurveData = []
        boxPlotCurveData = []
        ccn = 0
        if data.domain.classVar and data.domain.classVar.varType <> orange.VarTypes.Discrete:
            print "error, class variable not discrete:", data.domain.classVar
            return
        classes = data.domain.classVar.values if data.domain.classVar else ["(No class)"]
        allc = len(data.domain.classVar.values) if data.domain.classVar else 1
##        print data, len(data)
        for i, c in enumerate(classes):
            if progressBar <> None: progressBar(i*100.0/len(classes))
            classSymb = QwtSymbol(QwtSymbol.Ellipse, QBrush(self.classBrighterColor[i]), QPen(self.classBrighterColor[i]), QSize(7,7)) ##self.black
            self.showClasses.append(0)

            self.profileCurveKeys.append([])
            self.averageProfileCurveKeys.append([])
            allg = len(self.groups)
            gcn = 0
            grpcnx = 0
            for (grpname, grpattrs) in self.groups:
                oneClassData = data.select({data.domain.classVar.name:c}) if data.domain.classVar else data
                oneGrpData = oneClassData.select(orange.Domain(grpattrs, oneClassData.domain))

                ## single profiles
                nativeData = oneGrpData.native(2)
                yVals = [[] for cn in range(len(grpattrs))]
                alle = len(nativeData)
                ecn = 0
                for e, example in zip(nativeData, oneClassData):
                    ecn += 1
                    progress = 100.0*(ccn + (float(gcn)/allg) * (float(ecn)/alle))
                    progress = int(round(progress / allc))
                    if progressBar <> None: progressBar(progress)
                    y = []
                    x = []
                    xcn = grpcnx
                    vcn = 0
                    en = e.native(1)
                    for v in en:
                        if not v.isSpecial():
                            yVal = v.native()
                            yVals[vcn].append( yVal )
                            y.append( yVal )
                            x.append( xcn )
                        xcn += 1
                        vcn += 1
                    curve = self.addCurve('', style = QwtPlotCurve.Lines)
                    curve.example = example
                    if self.showAverageProfile:
                        curve.setPen(QPen(self.classColor[ccn], 1))
                    else:
                        curve.setPen(QPen(self.classBrighterColor[ccn], 1))
                    curve.setData(x, y)
                    curve.setSymbol(classSymb)
                    self.profileCurveKeys[-1].append(curve)

                ## average profile and box plot
                BPx = []
                BPy = []
                xcn = grpcnx
                vcn = 0
                dist = orange.DomainDistributions(oneGrpData)
                for a in dist:
                    if a and len(a) > 0:
                        ## box plot data
                        yavg = a.average()
                        yq1 = a.percentile(25)
                        yqM = a.percentile(50)
                        yq3 = a.percentile(75)

                        iqr = yq3 - yq1
                        yLowerCutOff = yq1 - 1.5 * iqr
                        yUpperCutOff = yq3 + 1.5 * iqr
                        
                        yVals[vcn].sort() 
                        ## find the smallest value above the lower inner fence
                        smallestAdjacentValue = None
                        for v in yVals[vcn]:
                            if v >= yLowerCutOff:
                                smallestAdjacentValue = v
                                break

                        yVals[vcn].reverse()
                        ## find the largest value below the upper inner fence
                        largestAdjacentValue = None
                        for v in yVals[vcn]:
                            if v <= yUpperCutOff:
                                largestAdjacentValue = v
                                break
                        BPy.append( largestAdjacentValue )
                        BPy.append( yq3 )
                        BPy.append( yavg )
                        BPy.append( yqM )
                        BPy.append( yq1 )
                        BPy.append( smallestAdjacentValue )
                        BPx.append( xcn )
                        BPx.append( xcn )
                        BPx.append( xcn )
                        BPx.append( xcn )
                        BPx.append( xcn )
                        BPx.append( xcn )

                    xcn += 1
                    vcn += 1

                boxPlotCurveData.append( (BPx, BPy, ccn) )
                grpcnx += len(grpattrs)
                gcn +=1
            ccn += 1

        for (x, y, tmpCcn) in boxPlotCurveData:
            classSymb = QwtSymbol(QwtSymbol.Cross, QBrush(self.classBrighterColor[tmpCcn]), QPen(self.classBrighterColor[tmpCcn]), QSize(8,8))
            curve = boxPlotQwtPlotCurve('', connectPoints = 1, tickXw = 1.0/5.0)
            curve.attach(self)
            curve.setPen(QPen(self.classBrighterColor[tmpCcn], 3))
            curve.setStyle(QwtPlotCurve.UserCurve)
            curve.setSymbol(classSymb)
            curve.setData(x, y)
            self.averageProfileCurveKeys[tmpCcn].append(curve)

        ## generate labels for attributes
        labels = []
        for (grpname, grpattrs) in self.groups:
            for a in grpattrs:
                labels.append( a.name)

        if None: 
            self.setXlabels(labels)
        self.updateCurveDisplay()

    def updateCurveDisplay(self):
        profilesCount = 0
        for cNum in range(len(self.showClasses or [1])):
            showCNum = (self.showClasses[cNum] <> 0) if len(self.showClasses) > 1 else True
##            print cNum, showCNum
            ## single profiles
            bSingle = showCNum and self.showSingleProfiles
            bAve = showCNum and self.showAverageProfile ## 1 = show average profiles for now
            for ckey in self.profileCurveKeys[cNum]:
                curve =  ckey #self.curve(ckey)
                if curve <> None:
                    curve.setVisible(bSingle)
                    profilesCount += 1 if bSingle else 0
##                    qp = self.curvePen(ckey)
                    qp = curve.pen()
                    if not(bAve):
                        qp.setColor(self.classBrighterColor[cNum])
                    else:
                        qp.setColor(self.classColor[cNum])
##                    curve.setPen(qp)

            ## average profiles
            for ckey in self.averageProfileCurveKeys[cNum]:
                curve =  ckey #self.curve(ckey)
                if curve <> None: curve.setVisible(bAve)

##        self.updateLayout()
        self.master.infoLabel.setText("Showing %i profiles" % profilesCount)
        self.setCurveRenderHints()
        self.replot()

    def setShowClasses(self, list):
        self.showClasses = list
        self.updateCurveDisplay()

    def setShowAverageProfile(self, v):
        self.showAverageProfile = v
        self.updateCurveDisplay()

    def setShowSingleProfiles(self, v):
        self.showSingleProfiles = v
        self.updateCurveDisplay()

    def setPointWidth(self, v):
        for cNum in range(len(self.showClasses)):
            for curve in self.profileCurveKeys[cNum]:
                symb = curve.symbol()
                symb.setSize(v, v)
##                curve.setSymbol(symb)
        self.replot()

    def setCurveWidth(self, v):
        fix = dict.fromkeys(self.selectedCurves, 2)
        fix[self.highlightedCurve] = 3
        for cNum in range(len(self.showClasses)):
            for curve in self.profileCurveKeys[cNum]:
                qp = curve.pen()
                qp.setWidth(v + fix.get(curve, 0))
##                curve.setPen(qp)
        self.curveWidth = v
        self.replot()

    def setAverageCurveWidth(self, v):
        for cNum in range(len(self.showClasses)):
            for curve in self.averageProfileCurveKeys[cNum]:
                qp = curve.pen()
                qp.setWidth(v)
##                curve.setPen(qp)
        self.replot()

    def setBoxPlotWidth(self, v):
        for cNum in range(len(self.showClasses)):
            for curve in self.averageProfileCurveKeys[cNum]:
##                c = self.curve(ckey)
                curve.setBoxPlotPenWidth(v)
        self.replot()

    def sizeHint(self):
        return QSize(170, 170)

    def closestCurve(self, point):
        pointDistances = [(curve,) + curve.closestPoint(point) for curve in self.itemList() if isinstance(curve, QwtPlotCurve) and curve.isVisible()]
        return min(pointDistances, key=lambda t:t[-1]) if pointDistances else (None, -1, sys.maxint)

    def curveUnderMouse(self, pos):
        pos = self.canvas().mapFrom(self, pos)
        curve, i, d = self.closestCurve(pos)
        if d <= 5 and hasattr(curve, "example"):
            return curve
        else:
            return None
    
    def mouseMoveEvent(self, event):
        curve = self.curveUnderMouse(event.pos())
        if self.highlightedCurve != curve:
            if self.highlightedCurve:
                self.highlightedCurve.setPen(QPen(self.highlightedCurve.pen().color(), self.curveWidth + (2 if self.highlightedCurve in self.selectedCurves else 0)))
            if curve:
                curve.setPen(QPen(curve.pen().color(), self.curveWidth + 3))
            self.highlightedCurve = curve
            self.replot()
            
            if curve:
                QToolTip.showText(event.globalPos(), "")
                QToolTip.showText(event.globalPos(), str(curve.example[self.master.profileLabel]))
                                         
        return OWGraph.mouseMoveEvent(self, event)
    
    def mousePressEvent(self, event):
        curve = self.curveUnderMouse(event.pos())
        if curve and event.button() == Qt.LeftButton and self.state == SELECT:
            if self.master.ctrlPressed:
                if curve in self.selectedCurves:
                    self.setSelectedCurves([c for c in self.selectedCurves if c is not curve])
                else:
                    self.setSelectedCurves(self.selectedCurves + [curve])
            else:
                self.setSelectedCurves([curve])
            self.replot()
                
        return OWGraph.mousePressEvent(self, event)
    
    def setCurveRenderHints(self):
        for item in self.itemList():
            if isinstance(item, QwtPlotCurve):
                item.setRenderHint(QwtPlotCurve.RenderAntialiased, self.renderAntialiased)
        self.replot()
        
    def setSelectedCurves(self, curves=[]):
        for c in self.selectedCurves:
            c.setPen(QPen(c.pen().color(), self.curveWidth))
        for c in curves:
            c.setPen(QPen(c.pen().color(), self.curveWidth + 2))
        self.selectedCurves = curves
        self.master.commitIf()
        
    def removeLastSelection(self):
        self.setSelectedCurves(self.selectedCurves[:-1])
        self.replot()
        
    def removeAllSelections(self):
        self.setSelectedCurves([])
        self.replot()
        
    def sendData(self):
        self.master.commitIf()


class OWDisplayProfiles(OWWidget):
    settingsList = ["SelectedClasses", "PointWidth", "CurveWidth", "AverageCurveWidth", "BoxPlotWidth", "ShowAverageProfile", "ShowSingleProfiles", "CutEnabled", "CutLow", "CutHigh", "autoSendSelected"]
    contextHandlers = {"": DomainContextHandler("", [ContextField("SelectedClasses")], loadImperfect=False)}
    def __init__(self, parent=None, signalManager = None):
#        self.callbackDeposit = [] # deposit for OWGUI callback functions
        OWWidget.__init__(self, parent, signalManager, 'Expression Profiles', 1)

        #set default settings
        self.ShowAverageProfile = 1
        self.ShowSingleProfiles = 0
        self.PointWidth = 2
        self.CurveWidth = 1
        self.AverageCurveWidth = 4
        self.BoxPlotWidth = 2
        self.SelectedClasses = []
        self.Classes = []
        self.autoSendSelected = 0
        self.selectionChangedFlag = False

        self.CutLow = 0; self.CutHigh = 0; self.CutEnabled = 0
        
        self.profileLabel = None

        #load settings
        self.loadSettings()

        # GUI
        self.graph = profilesGraph(self, self.mainArea, "")
        self.mainArea.layout().addWidget(self.graph)
        self.graph.hide()
        self.connect(self.graphButton, SIGNAL("clicked()"), self.graph.saveToFile)

        # GUI definition
        self.tabs = OWGUI.tabWidget(self.space)

        # GRAPH TAB
        GraphTab = OWGUI.createTabPage(self.tabs, "Graph")

        ## display options
        self.infoLabel = OWGUI.widgetLabel(OWGUI.widgetBox(GraphTab, "Info"), "No data on input.")
        displayOptBox = OWGUI.widgetBox(GraphTab, "Display") #QVButtonGroup("Display", GraphTab)
        displayOptButtons = ['Majority Class', 'Majority Class Probability', 'Target Class Probability', 'Number of Instances']
        OWGUI.checkBox(displayOptBox, self, 'ShowSingleProfiles', 'Expression Profiles', tooltip='', callback=self.updateShowSingleProfiles)
        OWGUI.checkBox(displayOptBox, self, 'ShowAverageProfile', 'Box Plot', tooltip='', callback=self.updateShowAverageProfile)

        ## class selection (classQLB)
        self.classQVGB = OWGUI.widgetBox(GraphTab, "Classes") #QVGroupBox(GraphTab)
##        self.classQVGB.setTitle("Classes")
        self.classQLB = OWGUI.listBox(self.classQVGB, self, "SelectedClasses", "Classes", selectionMode=QListWidget.MultiSelection, callback=self.classSelectionChange) #QListBox(self.classQVGB)
##        self.classQLB.setSelectionMode(QListBox.Multi)
        self.unselectAllClassedQLB = OWGUI.button(self.classQVGB, self, "Unselect all", callback=self.SUAclassQLB) #QPushButton("(Un)Select All", self.classQVGB)
##        self.connect(self.unselectAllClassedQLB, SIGNAL("clicked()"), self.SUAclassQLB)
##        self.connect(self.classQLB, SIGNAL("selectionChanged()"), self.classSelectionChange)

        ## show single/average profile
##        self.showAverageQLB = QPushButton("Box Plot", self.classQVGB)
##        self.showAverageQLB.setToggleButton(1)
##        self.showAverageQLB.setOn(self.ShowAverageProfile)
##        self.showSingleQLB = QPushButton("Single Profiles", self.classQVGB)
##        self.showSingleQLB.setToggleButton(1)
##        self.showSingleQLB.setOn(self.ShowSingleProfiles)
##        self.connect(self.showAverageQLB, SIGNAL("toggled(bool)"), self.updateShowAverageProfile)
##        self.connect(self.showSingleQLB, SIGNAL("toggled(bool)"), self.updateShowSingleProfiles)

##        self.tabs.insertTab(GraphTab, "Graph")

        # SETTINGS TAB
        SettingsTab = OWGUI.createTabPage(self.tabs, "Settings") #QVGroupBox(self)

        self.profileLabelComboBox = OWGUI.comboBox(SettingsTab, self, 'profileLabel', 'Profile Labels', sendSelectedValue=True, valueType=str)
        OWGUI.hSlider(SettingsTab, self, 'PointWidth', box='Point Width', minValue=0, maxValue=9, step=1, callback=self.updatePointWidth, ticks=1)
        OWGUI.hSlider(SettingsTab, self, 'CurveWidth', box='Profile Width', minValue=1, maxValue=9, step=1, callback=self.updateCurveWidth, ticks=1)
        OWGUI.hSlider(SettingsTab, self, 'AverageCurveWidth', box='Average Profile Width', minValue=1, maxValue=9, step=1, callback=self.updateAverageCurveWidth, ticks=1)
        OWGUI.hSlider(SettingsTab, self, 'BoxPlotWidth', box='Box Plot Width', minValue=0, maxValue=9, step=1, callback=self.updateBoxPlotWidth, ticks=1)
        OWGUI.checkBox(SettingsTab, self, 'graph.renderAntialiased', "Render antialiased", callback=self.graph.setCurveRenderHints)


    ## graph y scale
        box = OWGUI.widgetBox(SettingsTab, "Threshold/ Values") #QVButtonGroup("Threshol/ Values", SettingsTab)
        OWGUI.checkBox(box, self, 'CutEnabled', "Enabled", callback=self.setCutEnabled)
        self.sliderCutLow = OWGUI.qwtHSlider(box, self, 'CutLow', label='Low:', labelWidth=33, minValue=-20, maxValue=0, step=0.1, precision=1, ticks=0, maxWidth=80, callback=self.updateYaxis)
        self.sliderCutHigh = OWGUI.qwtHSlider(box, self, 'CutHigh', label='High:', labelWidth=33, minValue=0, maxValue=20, step=0.1, precision=1, ticks=0, maxWidth=80, callback=self.updateYaxis)
        if not self.CutEnabled:
            self.sliderCutLow.box.setDisabled(1)
            self.sliderCutHigh.box.setDisabled(1)

##        self.tabs.insertTab(SettingsTab, "Settings")

        self.toolbarSelection = ZoomSelectToolbar(self, self.controlArea, self.graph, self.autoSendSelected, buttons=(ZoomSelectToolbar.IconZoom, ZoomSelectToolbar.IconPan, ZoomSelectToolbar.IconSelect, ZoomSelectToolbar.IconSpace,
                                                                       ZoomSelectToolbar.IconRemoveLast, ZoomSelectToolbar.IconRemoveAll, ZoomSelectToolbar.IconSendSelection))
        cb = OWGUI.checkBox(self.controlArea, self, "autoSendSelected", "Auto send on selection change")
        OWGUI.setStopper(self, self.toolbarSelection.buttonSendSelections, cb, "selectionChangedFlag", self.commit)
        
        # inputs
        # data and graph temp variables
        
        self.inputs = [("Examples", ExampleTable, self.data, Default + Multiple)]
        self.outputs = [("Examples", ExampleTable, Default)]

        # temp variables
        self.MAdata = []
        self.classColor = None
        self.classBrighterColor = None
        self.numberOfClasses  = 0
        self.classValues = []
        self.ctrlPressed = False
        self.attrIcons = self.createAttributeIconDict()

        self.graph.canvas().setMouseTracking(1)

#        self.zoomStack = []
##        self.connect(self.graph,
##                     SIGNAL('plotMousePressed(const QMouseEvent&)'),
##                     self.onMousePressed)
##        self.connect(self.graph,
##                     SIGNAL('plotMouseReleased(const QMouseEvent&)'),
##                     self.onMouseReleased)
        self.resize(800,600)

    def updateYaxis(self):
        if (not self.CutEnabled):
            self.graph.setAxisAutoScale(QwtPlot.yLeft)
            self.graph.replot()
            return
                                                                                                                                     
        self.graph.setAxisScale(QwtPlot.yLeft, self.CutLow, self.CutHigh)
        self.graph.replot()

    def setCutEnabled(self):
        self.sliderCutLow.box.setDisabled(not self.CutEnabled)
        self.sliderCutHigh.box.setDisabled(not self.CutEnabled)
        self.updateYaxis()

#    def onMousePressed(self, e):
#        if Qt.LeftButton == e.button():
#            # Python semantics: self.pos = e.pos() does not work; force a copy
#            self.xpos = e.pos().x()
#            self.ypos = e.pos().y()
#            self.graph.enableOutline(1)
#            self.graph.setOutlinePen(QPen(Qt.black))
#            self.graph.setOutlineStyle(Qwt.Rect)
#            self.zooming = 1
#            if self.zoomStack == []:
#                self.zoomState = (
#                    self.graph.axisScale(QwtPlot.xBottom).lBound(),
#                    self.graph.axisScale(QwtPlot.xBottom).hBound(),
#                    self.graph.axisScale(QwtPlot.yLeft).lBound(),
#                    self.graph.axisScale(QwtPlot.yLeft).hBound(),
#                    )
#        elif Qt.RightButton == e.button():
#            self.zooming = 0
#        # fake a mouse move to show the cursor position
#
#    # onMousePressed()
#
#    def onMouseReleased(self, e):
#        if Qt.LeftButton == e.button():
#            xmin = min(self.xpos, e.pos().x())
#            xmax = max(self.xpos, e.pos().x())
#            ymin = min(self.ypos, e.pos().y())
#            ymax = max(self.ypos, e.pos().y())
#            self.graph.setOutlineStyle(Qwt.Cross)
#            xmin = self.graph.invTransform(QwtPlot.xBottom, xmin)
#            xmax = self.graph.invTransform(QwtPlot.xBottom, xmax)
#            ymin = self.graph.invTransform(QwtPlot.yLeft, ymin)
#            ymax = self.graph.invTransform(QwtPlot.yLeft, ymax)
#            if xmin == xmax or ymin == ymax:
#                return
#            self.zoomStack.append(self.zoomState)
#            self.zoomState = (xmin, xmax, ymin, ymax)
#            self.graph.enableOutline(0)
#        elif Qt.RightButton == e.button():
#            if len(self.zoomStack):
#                xmin, xmax, ymin, ymax = self.zoomStack.pop()
#            else:
#                self.graph.setAxisAutoScale(QwtPlot.xBottom)
#                self.graph.setAxisAutoScale(QwtPlot.yLeft)
#                self.graph.replot()
#                return
#
#        self.graph.setAxisScale(QwtPlot.xBottom, xmin, xmax)
#        self.graph.setAxisScale(QwtPlot.yLeft, ymin, ymax)
#        self.graph.replot()
#
#    def saveToFile(self):
#        qfileName = QFileDialog.getSaveFileName("graph.png","Portable Network Graphics (.PNG)\nWindows Bitmap (.BMP)\nGraphics Interchange Format (.GIF)", None, "Save to..")
#        fileName = str(qfileName)
#        if fileName == "": return
#        (fil,ext) = os.path.splitext(fileName)
#        ext = ext.replace(".","")
#        ext = ext.upper()
#        cl = 0
#        for g in self.graphs:
#            if g.isVisible():
#                clfname = fil + "_" + str(cl) + "." + ext
#                g.saveToFileDirect(clfname, ext)
#            cl += 1

    def updateShowAverageProfile(self):
        self.graph.setShowAverageProfile(self.ShowAverageProfile)

    def updateShowSingleProfiles(self):
        self.graph.setShowSingleProfiles(self.ShowSingleProfiles)

    def updatePointWidth(self):
        self.graph.setPointWidth(self.PointWidth)

    def updateCurveWidth(self):
        self.graph.setCurveWidth(self.CurveWidth)

    def updateAverageCurveWidth(self):
        self.graph.setAverageCurveWidth(self.AverageCurveWidth)

    def updateBoxPlotWidth(self):
        self.graph.setBoxPlotWidth(self.BoxPlotWidth)
        
    ##
    def selectUnselectAll(self, qlb):
        self.SelectedClasses = range(len(self.Classes)) if len(self.SelectedClasses) != len(self.Classes)  else []
        
        self.classSelectionChange()

    def SUAclassQLB(self):
        self.selectUnselectAll(self.classQLB)
    ##

    ## class selection (classQLB)
    def classSelectionChange(self):
##        list = []
##        selCls = []
##        for i in range(self.classQLB.count()):
##            if self.classQLB.isSelected(i):
##                list.append( 1 )
##                selCls.append(self.classValues[i])
##            else:
##                list.append( 0 )
        list = [1 if i in self.SelectedClasses else 0 for i in range(self.classQLB.count())]
        self.unselectAllClassedQLB.setText("Select all" if len(self.SelectedClasses) != len(self.Classes) else "Unselect all")
        selCls = [self.classValues[i] for i in self.SelectedClasses]
        self.graph.setShowClasses(list)
#        if selCls == []:
#            self.send("Examples", None)
#        else:
#            if len(self.MAdata) > 1:
#                newet = orange.ExampleTable(self.MAdata[0].domain)
#                for idx, i in enumerate(list):
#                    if i: newet.extend(self.MAdata[idx])
#            elif self.MAdata[0].domain.classVar:
#                newet = self.MAdata[0].filter({self.MAdata[0].domain.classVar.name:selCls})
#            else:
#                newet = self.MAdata[0]
#            self.send("Examples", newet)
    ##

    def calcGraph(self):
        ## compute from self.MAdata
        if len(self.MAdata) == 1:
            d = self.MAdata[0]
        elif len(self.MAdata) > 0:
            combCvals = [str(nd.name) for nd in self.MAdata]
            combClass = orange.EnumVariable('file', values=combCvals)
            combDomain = orange.Domain(self.MAdata[0].domain.attributes + [combClass])
            d = orange.ExampleTable(combDomain)
            
            for tmpd in self.MAdata:
                newtmpd = tmpd.select(combDomain)
                for te in newtmpd:
                    te.setclass(tmpd.name)
                d.extend(newtmpd)
        else:
            return

        self.progressBarInit()
        self.graph.setData(d, self.classColor, self.classBrighterColor, self.ShowAverageProfile, self.ShowSingleProfiles, self.progressBarSet)
        self.graph.setPointWidth(self.PointWidth)
        self.graph.setCurveWidth(self.CurveWidth)
        self.graph.setAverageCurveWidth(self.AverageCurveWidth)
        self.graph.setBoxPlotWidth(self.BoxPlotWidth)

        self.graph.setAxisAutoScale(QwtPlot.xBottom)
##        self.graph.setAxisAutoScale(QwtPlot.yLeft)
        self.updateYaxis()
        self.progressBarFinished()

    def newdata(self):
        self.classQLB.clear()
        self.profileLabelComboBox.clear()
##        if len(self.MAdata) > 1 or (len(self.MAdata) == 1 and self.MAdata[0].domain.classVar.varType == orange.VarTypes.Discrete):
        if len(self.MAdata) >= 1:
            ## classQLB
            if len(self.MAdata) == 1 and self.MAdata[0].domain.classVar and self.MAdata[0].domain.classVar.varType == orange.VarTypes.Discrete:
                self.numberOfClasses = len(self.MAdata[0].domain.classVar.values)
            else:
                self.numberOfClasses = len(self.MAdata)
            self.classColor = ColorPaletteGenerator(self.numberOfClasses)
            self.classBrighterColor = [self.classColor[c, 160] for c in range(self.numberOfClasses)]

            self.calcGraph()
            ## update graphics
            ## classQLB
            self.classQVGB.show()
            if len(self.MAdata) == 1 and self.MAdata[0].domain.classVar and self.MAdata[0].domain.classVar.varType == orange.VarTypes.Discrete:
                self.classValues = self.MAdata[0].domain.classVar.values.native()
            else:
                self.classValues = [str(nd.name) for nd in self.MAdata]

##            selection = QItemSelection()
##            for cn in range(len(self.classValues)):
##                item = QListWidgetItem(QIcon(ColorPixmap(self.classBrighterColor[cn])), self.classValues[cn])
##                self.classQLB.addItem(item)
##                selection.select(self.classQLB.indexFromItem(item), self.classQLB.indexFromItem(item))
##            self.classQLB.selectionModel().select(selection, QItemSelectionModel.Select)
            self.Classes = [(QIcon(ColorPixmap(self.classBrighterColor[cn])), self.classValues[cn])  for cn in range(len(self.classValues))]
            self.SelectedClasses = range(len(self.Classes))
            
##            self.classQLB.selectAll()  ##or: if numberOfClasses > 0: self.classQLB.setSelected(0, 1)
##            self.update()

##            if len(self.MAdata) == 1 and self.MAdata[0].noclass:
##                pass
            self.classQVGB.setDisabled(len(self.MAdata) == 1 and self.MAdata[0].noclass)
            attrs = self.MAdata[0].domain.variables + self.MAdata[0].domain.getmetas().values()
            for attr in attrs:
                self.profileLabelComboBox.addItem(self.attrIcons[attr.varType], attr.name)
            stringAttrs = [attr for attr in attrs if attr.varType == orange.VarTypes.String]
            discAttrs = [attr for attr in attrs if attr.varType == orange.VarTypes.Discrete]
            attrs = stringAttrs[-1:] + discAttrs[-1:] + list(attrs[:1])
            self.profileLabel = attrs[0].name
        else:
            self.classColor = None
            self.classBrighterColor = None
            self.classValues = []
##        self.classQVGB.update()
##        self.classQVGB.layout().activate()
        self.graph.show()
##        self.layout.activate() # this is needed to scale the widget correctly

    def data(self, MAdata, id=None):
        ## if there is no class attribute, create a dummy one
        self.closeContext()
        if MAdata and MAdata.domain.classVar == None:
##            noClass = orange.EnumVariable('file', values=['n', 'y'])
##            newDomain = orange.Domain(MAdata.domain.attributes + [noClass])
##            mname = MAdata.name ## remember name 
##            MAdata = MAdata.select(newDomain) ## because select forgets it
##            MAdata.name = mname
##            for e in MAdata: e.setclass('n')
            MAdata.setattr("noclass", 1) ## remember that there is no class to display
        elif MAdata and MAdata.domain.classVar.varType <> orange.VarTypes.Discrete:
            print "error, ignoring table, because its class variable not discrete:", MAdata.domain.classVar
##            MAdata = None
            MAdata.setattr("noclass", 1) ## remember that there is no class to display
        elif MAdata:
            MAdata.setattr("noclass", 0) ## there are classes by default

        ## handling of more than one data set
        # check if the same domain
        if MAdata and len(self.MAdata) and str(MAdata.domain.attributes) <> str(self.MAdata[0].domain.attributes):
            print "domains:", MAdata.domain.attributes
            print "and,   :", self.MAdata[0].domain.attributes
            print "are not same"
##            MAdata = None

        ids = [d.id for d in self.MAdata]
        if not MAdata:
            if id in ids:
                del self.MAdata[ids.index(id)]
        else:
            MAdata.setattr("id", id)
            if id in ids:
                MAdata.setattr("id", id)
                indx = ids.index(id)
                self.MAdata[indx] = MAdata
            else:
                self.MAdata.append(MAdata)

        if len(self.MAdata) == 0:
            print "hiding graph"
            self.graph.hide()
            self.classQVGB.hide()
            self.infoLabel.setText("No data on input")
            return

        self.newdata()
        self.openContext("", MAdata)
        self.classSelectionChange()

    def sendReport(self):
        self.startReport("%s" % self.windowTitle())
        self.reportSettings("Settings", ([("Selected classes" , ",".join(self.Classes[i][1] for i in self.SelectedClasses))] if len(self.Classes) > 1 else []) +\
                                        [("Show box plot", self.ShowAverageProfile),
                                         ("Show profiles", self.ShowSingleProfiles)])
        self.reportRaw("<p>%s</p>" % self.infoLabel.text())
        self.reportImage(lambda *x: OWChooseImageSizeDlg(self.graph).saveImage(*x))
        
    def keyPressEvent(self, event):
        self.ctrlPressed = event.key() == Qt.Key_Control
        return OWWidget.keyPressEvent(self, event)
    
    def keyReleaseEvent(self, event):
        self.ctrlPressed = self.ctrlPressed and not event.key() == Qt.Key_Control
        return OWWidget.keyReleaseEvent(self, event)
    
    def commitIf(self):
        if self.autoSendSelected:
            self.commit()
        else:
            self.selectionChangedFlag = True
            
    def commit(self):
        data = [c.example for c in self.graph.selectedCurves if hasattr(c, "example")]
        if data:
            data = orange.ExampleTable(data)
        else:
            data = None
            
        self.send("Examples", data)
        self.selectionChangedFlag = False
    
# following is not needed, data handles these cases
##    def cdata(self, MAcdata):
##        if not MAcdata:
##            self.graph.hide()
##            return
##        self.MAdata = MAcdata
##        self.MAnoclass = 0
##        self.newdata()

if __name__ == "__main__":
    a = QApplication(sys.argv)
    owdm = OWDisplayProfiles()
##    a.setMainWidget(owdm)
##    d = orange.ExampleTable('e:\\profiles')
##    d = orange.ExampleTable('e:\\profiles-classes')
    d = orange.ExampleTable('../../../doc/datasets/brown-selected')
    print len(d)
    owdm.data(d)
    owdm.show()
    a.exec_()
    owdm.saveSettings()



from __future__ import absolute_import

import random
import math
from collections import defaultdict

import scipy.stats
import scipy.special
import numpy
import Orange, Orange.utils, statc

from .. import utils
obiExpression = utils.expression

def corgs_activity_score(ex, corg):
    """ activity score for a sample for pathway given by corgs """
    #print [ ex[i].value for i in corg ] #FIXME what to do with unknown values?
    return sum(ex[i].value if ex[i].value != '?' else 0.0 for i in corg)/len(corg)**0.5


def PLSCall(data, y=None, x=None, nc=None, weight=None, save_partial=False):

    def normalize(vector):
        return vector / numpy.linalg.norm(vector)

    if y == None:
        y = [ data.domain.classVar ]
    if x == None:
        x = [v for v in data.domain.variables if v not in y]

    Ncomp = nc if nc is not None else len(x)
        
    dataX = Orange.data.Table(Orange.data.Domain(x, False), data)
    dataY = Orange.data.Table(Orange.data.Domain(y, False), data)

    # transformation to numpy arrays
    X = dataX.toNumpy()[0]
    Y = dataY.toNumpy()[0]

    # data dimensions
    n, mx = numpy.shape(X)
    my = numpy.shape(Y)[1]

    # Z-scores of original matrices
    YMean = numpy.mean(Y, axis = 0)
    XMean = numpy.mean(X, axis = 0)
    
    X = (X-XMean)
    Y = (Y-YMean)

    P = numpy.empty((mx,Ncomp))
    T = numpy.empty((n,Ncomp))
    W = numpy.empty((mx,Ncomp))
    E,F = X,Y

    dot = numpy.dot
    norm = numpy.linalg.norm

    #PLS1 - from Gutkin, shamir, Dror: SlimPLS

    for i in range(Ncomp):
        w = dot(E.T,F)
        w = w/norm(w) #normalize w in Gutkin et al the do w*c, where c is 1/norm(w)
        t = dot(E, w) #t_i -> a row vector
        p = dot(E.T, t)/dot(t.T, t) #p_i t.T is a row vector - this is inner(t.T, t.T)
        q = dot(F.T, t)/dot(t.T, t) #q_i
            
        E = E - dot(t, p.T)
        F = F - dot(t, q.T)

        T[:,i] = t.T
        W[:,i] = w.T
        P[:,i] = p.T

    return XMean, W, P, T


def pca(M, snapshot=None):
    "Perform PCA on M, return eigenvectors and eigenvalues, sorted."
    XMean = numpy.mean(M, axis = 0)
    M = M - XMean
    
    if snapshot == None:
        snapshot = M.shape[0] < M.shape[1]

    if snapshot: #less columns than rows
        evals, evecsC = numpy.linalg.eigh(M.dot(M.T)) #columns of evecsC are eigenvectors
        evecs = M.T.dot(evecsC)/numpy.sqrt(numpy.abs(evals))
    else:
        evals, evecs = numpy.linalg.eigh(M.T.dot(M))
    
    evecs = evecs.T

    # sort the eigenvalues and eigenvectors, decending order
    order = (numpy.argsort(numpy.abs(evals))[::-1])
    evecs = numpy.take(evecs, order, 0)
    evals = numpy.take(evals, order)

    return evals, evecs, XMean

def pca2(M):
    """ Perform PCA on M, return eigenvectors and eigenvalues, sorted.
    Mostly euivalent to pca(), slightly slower but more stable."""
    XMean = numpy.mean(M, axis = 0)
    M = M - XMean
    U,s,Vt = numpy.linalg.svd(M, full_matrices=False)
    return s*s, Vt, XMean
    
class GeneSetTrans(object):

    __new__ = Orange.utils._orange__new__(object)

    def _match_instance(self, instance, geneset, takegenes=None):
        """
        Return
            - a gene matcher with the instance as a target
            - { name: attribute indices } of an instance
            - genes names on the data set that were matched by the gene set

        If takegenes is a list of indices, use only genes from
        the gene set with specified indices.
        """
        nm, name_ind = mat_ni(instance.domain, self.matcher)
        genes = [ nm.umatch(gene) for gene in geneset ]
        if takegenes:
            genes = [ genes[i] for i in takegenes ]
        return nm, name_ind, genes

    def _match_data(self, data, geneset, odic=False):
        nm, name_ind = mat_ni(data.domain, self.matcher)
        genes = [ nm.umatch(gene) for gene in geneset ]
        if odic:
            to_geneset = dict(zip(genes, geneset))
        takegenes = [ i for i,a in enumerate(genes) if a != None ]
        genes = [ genes[i] for i in takegenes ]
        if odic:
            return nm, name_ind, genes, takegenes, to_geneset
        else:
            return nm, name_ind, genes, takegenes

    def __init__(self, matcher=None, gene_sets=None, min_size=3, max_size=1000, min_part=0.1, class_values=None, cv=False):
        self.matcher = matcher
        self.gene_sets = gene_sets
        self.min_size = min_size
        self.max_size = max_size
        self.min_part = min_part
        self.class_values = class_values
        self._cache = {}
        self.cv = cv

    def __call__(self, data, weight_id=None):

        from .. import gsea as obiGsea
        #selection of classes and gene sets
        data = obiGsea.takeClasses(data, classValues=self.class_values)
        nm,_ =  mat_ni(data.domain, self.matcher)
        gene_sets = select_genesets(nm, self.gene_sets, self.min_size, self.max_size, self.min_part)

        #build a new domain
        #print "WHOLE"
        newfeatures = self.build_features(data, gene_sets)
        newdomain = Orange.data.Domain(newfeatures, data.domain.class_var)

        #build a data set with cross validation
        if self.cv == False:
            return Orange.data.Table(newdomain, data)
        else:
            # The domain has the transformer that is build on all samples,
            # while the transformed data table uses cross-validation
            # internally
            if self.cv == True:
                cvi = Orange.data.sample.SubsetIndicesCV(data, 5)
            elif self.cv != False:
                cvi = self.cv(data)
            data_cv = [ [] for _ in range(len(data)) ]
            for f in set(cvi):
                #print "FOLD", f
                learn = data.select(cvi, f, negate=True)
                test = data.select(cvi, f)
                lf = self.build_features(learn, gene_sets)
                transd = Orange.data.Domain(lf, data.domain.class_var)
                trans_test = Orange.data.Table(transd, test)
                for ex, pos in \
                    zip(trans_test, [ i for i,n in enumerate(cvi) if n == f ]):
                    data_cv[pos] = ex.native(0)
            return Orange.data.Table(newdomain, data_cv)

    def build_features(self, data, gene_sets):
        return [ self.build_feature(data, gs) for gs in gene_sets ]

def normcdf(x, mi, st):
    #implementation with scipy is almost the same as from Gary's stats
    #return 0.5*(2. - stats.erfcc((x - mi)/(st*math.sqrt(2))))
    return 0.5*(2. - scipy.special.erfc((x - mi)/(st*math.sqrt(2))))

class AT_edelmanParametric(object):

    def __init__(self, **kwargs):
        for a,b in kwargs.items():
            setattr(self, a, b)

    def __call__(self, nval):

        if self.mi1 == None or self.mi2 == None or self.st1 == None or self.st2 == None:
            return 0.0 

        val = nval

        try:
            if val >= self.mi1:
                p1 = 1 - normcdf(val, self.mi1, self.st1)
            else:
                p1 = normcdf(val, self.mi1, self.st1)

            if val >= self.mi2:
                p2 = 1 - normcdf(val, self.mi2, self.st2)
            else:
                p2 = normcdf(val, self.mi2, self.st2)

            #print p1, p2
            return math.log(p1/p2)
        except:
            #print p1, p2, "exception"
            return 0

def estimate_gaussian_per_class(data, i, a=None, b=None, common_if_extreme=False):
    cv = data.domain.class_var

    if a == None: a = cv.values[0]
    if b == None: b = cv.values[1]

    def avWCVal(value):
        return [ex[i].value for ex in data if ex[-1].value == value and not ex[i].isSpecial() ]

    list1 = avWCVal(a)
    list2 = avWCVal(b)

    mi1 = mi2 = st1 = st2 = None

    try:
        mi1 = statc.mean(list1)
        st1 = statc.std(list1)
    except:
        pass

    try:
        mi2 = statc.mean(list2)
        st2 = statc.std(list2)
    except:
        pass

    def extreme():
        return st1 == 0 or st2 == 0
    
    if common_if_extreme and extreme():
        st1 = st2 = statc.std(list1 + list2)

    return mi1, st1, mi2, st2

class AT_edelmanParametricLearner(object):
    """
    Returns attribute transfromer for Edelman parametric measure for a
    given attribute in the dataset.  Edelman et al, 06. Modified a bit.
    """

    def __init__(self, a=None, b=None):
        """
        a and b are choosen class values.
        """
        self.a = a
        self.b = b

    def __call__(self, i, data):
        cv = data.domain.class_var

        if self.a == None: self.a = cv.values[0]
        if self.b == None: self.b = cv.values[1]

        mi1, st1, mi2, st2 = estimate_gaussian_per_class(data, i, a=self.a, b=self.b)

        return AT_edelmanParametric(mi1=mi1, mi2=mi2, st1=st1, st2=st2)

class AT_loess(object):

    def __init__(self, **kwargs):
        for a,b in kwargs.items():
            setattr(self, a, b)

    def __call__(self, nval):

        val = nval

        def saveplog(a,b):
            try:
                return math.log(a/b)
            except:
                if a < b:
                    return -10
                else:
                    return +10

        try:
            ocene = self.condprob(val)
            if sum(ocene) < 0.01:
                return 0.0
            return saveplog(ocene[0], ocene[1])

        except:
            return 0.0

class AT_loessLearner(object):

    def __call__(self, i, data):
        try:
            ca = Orange.statistics.contingency.VarClass(data.domain.attributes[i], data)
            a =  Orange.statistics.estimate.ConditionalLoess(ca, nPoints=5)
            return AT_loess(condprob=a)
        except:
            return AT_loess(condprob=None)

def nth(l, n):
    return [a[n] for a in l]

class Assess(GeneSetTrans):
    """
    Uses the underlying GSEA code to select genes.
    Takes data and creates attribute transformations.
    """

    def __init__(self, rankingf=None, **kwargs):
        self.rankingf = rankingf
        if self.rankingf == None:
            self.rankingf = AT_edelmanParametricLearner()
        self.example_buffer = {}
        self.attransv = 0
        self.ignore_unmatchable_context = True
        super(Assess, self).__init__(**kwargs)

    def _ordered_and_lcor(self, ex, nm, name_ind, attrans, attransv):
        """ Buffered! It should be computed only once per example. """ 
        #name_ind and nm are always co-created, so I need to have only one as a key
        key = (ex, nm, attransv)
        from .. import gsea as obiGsea
        if key not in self.example_buffer:
            ex_atts = [ at.name for at in ex.domain.attributes ]
            new_atts = [ name_ind[nm.umatch(an)] if nm.umatch(an) != None else (None if self.ignore_unmatchable_context else i)
                for i,an in enumerate(ex_atts) ]

            #new_atts: indices of genes in original data for that sample 
            #POSSIBLE REVERSE IMPLEMENTATION (slightly different
            #for data from different chips):
            #save pairs together and sort (or equiv. dictionary transformation)

            indexes = filter(lambda x: x[0] != None, zip(new_atts, range(len(ex_atts))))

            lcor = [ attrans[index_in_data](ex[index_in_ex].value) 
                for index_in_data, index_in_ex in indexes if
                ex[index_in_ex].value != '?' ]

            indices_to_lcori = dict( (index_in_ex, i) for i,(_, index_in_ex) in enumerate(indexes) 
                if ex[index_in_ex].value != '?')

            #indexes in original lcor, sorted from higher to lower values
            ordered = obiGsea.orderedPointersCorr(lcor)
            rev2 = numpy.argsort(ordered)
            self.example_buffer[key] = lcor, ordered, rev2, indices_to_lcori
        return self.example_buffer[key]

    def build_features(self, data, gene_sets):

        from .. import gsea as obiGsea

        attributes = []

        #attrans: { i_orig: ranking_function }
        attrans = [ self.rankingf(iat, data) for iat, at in enumerate(data.domain.attributes) ]
        attransv = self.attransv
        self.attransv += 1

        nm_all, _ =  mat_ni(data.domain, self.matcher)

        for gs in gene_sets:

            at = Orange.feature.Continuous(name=str(gs))

            geneset = list(gs.genes)
            nm, name_ind, genes, takegenes, to_geneset = self._match_data(data, geneset, odic=True)
            takegenes = [ geneset[i] for i in takegenes ]
            genes = set(genes)

            def t(ex, w, takegenes=takegenes, nm=nm, attrans=attrans, attransv=attransv):
                nm2, name_ind2, genes2 = self._match_instance(ex, takegenes)
                lcor, ordered, rev2, indices_to_lcori = \
                    self._ordered_and_lcor(ex, nm, name_ind, attrans, attransv)

           
                #subset = list of indices, lcor = correlations, ordered = order
                #make it compatible with lcor, if some are missing in lcor
                subset = filter(None,
                    [ indices_to_lcori.get(name_ind2[g], None) for g in genes2 ] )

                return obiGsea.enrichmentScoreRanked(subset, lcor, ordered, rev2=rev2)[0] 

            at.get_value_from = t
            attributes.append(at)

        return attributes
   
def setSig_example_geneset(ex, data, no_unknowns, check_same=False):
    """ Gets learning data and example with the same domain, both
    containing only genes from the gene set. """

    distances = [ [], [] ]    

    def pearson(ex1, ex2):
        vals1 = ex1.native(0)[:-1]
        vals2 = ex2.native(0)[:-1]

        if check_same and vals1 == vals2:
            return 10 #they are the same

        #leaves undefined elements out
        if not no_unknowns:
            common = [ True if v1 != "?" and v2 != "?" else False \
                for v1,v2 in zip(vals1,vals2) ]
            vals1 = [ v for v,c in zip(vals1, common) if c ]
            vals2 = [ v for v,c in zip(vals2, common) if c ]

        #statc correlation is from 5-10 times faster than numpy!
        try:
            return statc.pearsonr(vals1, vals2)[0]
        except:
            return numpy.corrcoef([vals1, vals2])[0,1] 
        

    def ttest(ex1, ex2):
        try:
            return float(scipy.stats.ttest_ind(ex1, ex2)[0])
        except:
            return 0.0
    
    #maps class value to its index
    classValueMap = dict( [ (val,i) for i,val in enumerate(data.domain.class_var.values) ])
 
    #create distances to all learning data - save or other class

    for c in data:
        p = pearson(c, ex)
        if p != 10:
             distances[classValueMap[c[-1].value]].append(pearson(c, ex))

    return ttest(distances[0], distances[1])

@Orange.utils.lru_cache(maxsize=10)
def mat_ni(domain, matcher):
    """ Return (in a tuple):
        - a gene matcher that matches to the attribute names of data
        - a dictionary attribute names -> indices in the data set.
    """
    nm = matcher([at.name for at in domain.attributes])
    name_ind = dict((n.name,i) for i,n in enumerate(domain.attributes))
    return nm, name_ind

def select_genesets(nm, gene_sets, min_size=3, max_size=1000, min_part=0.1):
    """ Returns a list of gene sets that have sizes in limits """

    def ok_sizes(gs):
        """compares sizes of genesets to limitations"""
        transl = filter(lambda x: x != None, [ nm.umatch(gene) for gene in gs.genes ])
        if len(transl) >= min_size \
            and len(transl) <= max_size \
            and float(len(transl))/len(gs.genes) >= min_part:
            return True
        return False

    return filter(ok_sizes, gene_sets) 

def vou(ex, gn, indices):
    """ returns the value or "?" for the given gene name gn"""
    if gn not in indices:
        return "?"
    else:
        return ex[indices[gn]].value

class SetSig(GeneSetTrans):

    def __init__(self, **kwargs):
        self.no_unknowns = kwargs.pop("no_unknowns", False)
        self.check_same = kwargs.pop("check_same", False)
        super(SetSig, self).__init__(**kwargs)

    def build_feature(self, data, gs):

        at = Orange.feature.Continuous(name=str(gs))
        geneset = list(gs.genes)
        nm, name_ind, genes, takegenes = self._match_data(data, geneset)
        indices = [ name_ind[gene] for gene in genes ]
        takegenes = [ geneset[i] for i in takegenes ]

        def t(ex, w, gs=gs, data=data, indices=indices, takegenes=takegenes):
            nm2, name_ind2, genes2 = self._match_instance(ex, takegenes)

            domain = Orange.data.Domain([data.domain.attributes[i] for i in indices], data.domain.class_var)
            datao = Orange.data.Table(domain, data)
           
            #convert the example to the same domain
            exvalues = [ vou(ex, gn, name_ind2) for gn in genes2 ] + [ "?" ]
            example = Orange.data.Instance(domain, exvalues)

            return setSig_example_geneset(example, datao, self.no_unknowns, check_same=self.check_same) #only this one is setsig specific
     
        at.get_value_from = t
        return at

class ParametrizedTransformation(GeneSetTrans):

    def _get_par(self, datao):
        """ Get parameters for a subset of data, that comprises only the gene set """
        pass
        
    def _use_par(self, ex, constructt):
        pass
    
    def build_feature(self, data, gs):

        at = Orange.feature.Continuous(name=str(gs))

        geneset = list(gs.genes)
        nm, name_ind, genes, takegenes = self._match_data(data, geneset)
        domain = Orange.data.Domain([data.domain.attributes[name_ind[gene]] for gene in genes], data.domain.class_var)
        datao = Orange.data.Table(domain, data)
        takegenes = [ geneset[i] for i in takegenes ]

        constructt = self._get_par(datao)

        def t(ex, w, constructt=constructt, takegenes=takegenes, domain=domain):
            nm2, name_ind2, genes2 = self._match_instance(ex, takegenes)
          
            #convert the example to the same domain
            exvalues = [ vou(ex, gn, name_ind2) for gn in genes2 ] + [ "?" ]
            ex = Orange.data.Instance(domain, exvalues)

            return self._use_par(ex, constructt)
        
        at.get_value_from = t
        at.dbg = constructt #for debugging
        
        return at

class PLS(ParametrizedTransformation):

    def _get_par(self, datao):
        return PLSCall(datao, nc=1, y=[datao.domain.class_var])
        
    def _use_par(self, ex, constructt):
        ex = [ ex[i].value for i in range(len(ex.domain.attributes)) ]
        xmean, W, P, _ = constructt
        ex = ex - xmean # same input transformation

        nc = W.shape[1]

        TR = numpy.empty((1, nc))
        XR = ex

        dot = numpy.dot

        for i in range(nc):
           t = dot(XR, W[:,i].T)
           XR = XR - t*numpy.array([P[:,i]])
           TR[:,i] = t

        return TR[0][0]
 
def eigvturn(A):
    """ It multiplies rows (vectors of unit lengths) where 
        sum < 0 with -1. """
    turn = (numpy.sum(A, axis=1, keepdims=True) > 0)*2 - 1
    return A*turn

class PCA(ParametrizedTransformation):

    def __init__(self, **kwargs):
        self.turn = kwargs.pop("turn", False) #turn eigenvetors
        if self.turn == True:
            self.turn = eigvturn
        super(PCA, self).__init__(**kwargs)

    def _get_par(self, datao):
        M = datao.toNumpy("a")[0]
        evals, evect, xmean = pca(M)
        if self.turn:
            evect = self.turn(evect)
        return evals, evect, xmean

    def _use_par(self, arr, constructt):
        arr = [ arr[i].value for i in range(len(arr.domain.attributes)) ]
        evals, evect, xmean = constructt

        arr = arr - xmean # same input transformation - a row in a matrix
        ev0 = evect[0] #this is a row in a matrix - do a dot product
        a = numpy.dot(arr, ev0)

        return a

class SimpleFun(GeneSetTrans):

    def build_feature(self, data, gs):

        at = Orange.feature.Continuous(name=str(gs))

        def t(ex, w, gs=gs):
            geneset = list(gs.genes)
            nm2, name_ind2, genes2 = self._match_instance(ex, geneset)
           
            exvalues = [ vou(ex, gn, name_ind2) for gn in genes2 ] + [ "?" ]
            exvalues = filter(lambda x: x != "?", exvalues)

            return self.fn(exvalues)
     
        at.get_value_from = t
        return at

class Mean(SimpleFun):

    def __init__(self, **kwargs):
       self.fn = numpy.mean
       super(Mean, self).__init__(**kwargs)

class Median(SimpleFun):

    def __init__(self, **kwargs):
       self.fn = numpy.median
       super(Median, self).__init__(**kwargs)

class GSA(GeneSetTrans):

    def build_features(self, data, gene_sets):

        attributes = []

        def tscorec(data, at, cache=None):
            ma = obiExpression.MA_t_test()(at,data)
            return ma

        tscores = [ tscorec(data, at) for at in data.domain.attributes ]

        def to_z_score(t):
            return float(scipy.stats.norm.ppf(scipy.stats.t.cdf(t, len(data)-2)))

        zscores = map(to_z_score, tscores)

        for gs in gene_sets:

            at = Orange.feature.Continuous(name=str(gs))

            geneset = list(gs.genes)
            nm, name_ind, genes, takegenes, to_geneset = self._match_data(data, geneset, odic=True)
            #take each gene only once
            genes = set(genes)

            D = numpy.mean([max(zscores[name_ind[g]],0) for g in genes]) \
                + numpy.mean([min(zscores[name_ind[g]],0) for g in genes])

            if D >= 0:
                consider_genes = [ to_geneset[g] for g in genes if zscores[name_ind[g]] > 0.0 ]
            else:
                consider_genes = [ to_geneset[g] for g in genes if zscores[name_ind[g]] < 0.0 ]

            def t(ex, w, consider_genes=consider_genes):
                nm2, name_ind2, genes2 = self._match_instance(ex, consider_genes)
              
                #convert the example to the same domain
                exvalues = [ vou(ex, gn, name_ind2) for gn in genes2 ] + [ "?" ]
                exvalues = filter(lambda x: x != "?", exvalues)
              
                return numpy.mean(exvalues)

            at.get_value_from = t
            attributes.append(at)

        return attributes

def tscorec(data, at, cache=None):
    """ Cached attribute  tscore calculation """
    if cache != None and at in cache: return cache[at]
    ma = obiExpression.MA_t_test()(at,data)
    if cache != None: cache[at] = ma
    return ma

def nth(l, n):
    return [a[n] for a in l]

def compute_corg(data, inds, tscorecache):
    """
    Compute CORG for this geneset specified with gene inds
    in the example table. Output is the list of gene inds
    in CORG.

    """
    #order member genes by their t-scores: decreasing, if av(t-score) >= 0,
    #else increasing
    tscores = [ tscorec(data, at, tscorecache) for at in inds ]
    sortedinds = nth(sorted(zip(inds,tscores), key=lambda x: x[1], \
        reverse=numpy.mean(tscores) >= 0), 0)

    def S(corg):
        """ Activity score separation - S(G) in 
        the article """
        asv = Orange.feature.Continuous(name='AS')
        asv.getValueFrom = lambda ex,rw: Orange.data.Value(asv, corgs_activity_score(ex, corg))
        data2 = Orange.data.Table(Orange.data.Domain([asv], data.domain.classVar), data)
        return abs(tscorec(data2, 0)) #FIXME absolute - nothing in the article abs()
            
    #greedily find CORGS procing the best separation
    g = S(sortedinds[:1])
    bg = 1
    for a in range(2, len(sortedinds)+1):
        tg = S(sortedinds[:a])
        if tg > g: #improvement
            g = tg
            bg = a
        else:
            break
        
    return sortedinds[:bg]

class CORGs(ParametrizedTransformation):
    """
    WARNING: input has to be z_ij table! each gene needs to be normalized
    (mean=0, stdev=1) for all samples.
    """

    def build_features(self, *args, **kwargs):
        self.tscorecache = {} #reset a cache
        return super(CORGs, self).build_features(*args, **kwargs)

    def build_feature(self, data, gs):

        at = Orange.feature.Continuous(name=str(gs))
        geneset = list(gs.genes)

        nm, name_ind, genes, takegenes, to_geneset = self._match_data(data, geneset, odic=True)
        indices = compute_corg(data, [ name_ind[g] for g in genes ], self.tscorecache)

        ind_names = dict( (a,b) for b,a in name_ind.items() )
        selected_genes = sorted(set([to_geneset[ind_names[i]] for i in indices]))
    
        def t(ex, w, corg=selected_genes): #copy od the data
            nm2, name_ind2, genes2 = self._match_instance(ex, corg, None)
            exvalues = [ vou(ex, gn, name_ind2) for gn in genes2 ]
            return sum(v if v != '?' else 0.0 for v in exvalues)/len(corg)**0.5
     
        at.get_value_from = t
        return at

def compute_llr(data, inds, cache):
    """
    Compute CORG for this geneset specified with gene inds
    in the example table. Output is the list of gene inds
    in CORG.
    """
    def gaussianc(data, at, cache=None):
        """ Cached attribute  tscore calculation """
        if cache != None and at in cache: return cache[at]
        ma = estimate_gaussian_per_class(data, at, common_if_extreme=True)
        if cache != None: cache[at] = ma
        return ma

    gf = [ gaussianc(data, at, cache) for at in inds ]
    return gf

""" To avoid scipy overhead """
from math import pi
_norm_pdf_C = math.sqrt(2*pi)
_norm_pdf_logC = math.log(_norm_pdf_C)
def _norm_logpdf(x, mi, std):
    return -(x-mi)**2 / (2.0*std**2) - _norm_pdf_logC - math.log(std)

def _llrlogratio(v, mi1, std1, mi2, std2):
    if mi1 == None or std1 == None or mi2 == None or std2 == None or std1 == 0 or std2 == 0:
        return 0. #problem with estimation
    #lpdf1 = scipy.stats.norm.logpdf(v, mi1, std1)
    #lpdf2 = scipy.stats.norm.logpdf(v, mi2, std2)
    lpdf1 = _norm_logpdf(v, mi1, std1) #avoids scipy's overhead
    lpdf2 = _norm_logpdf(v, mi2, std2)
    r = lpdf1 - lpdf2
    return r

class LLR(ParametrizedTransformation):
    """ 
    From Su et al: Accurate and Reliable Cancer Classification Base
    on Probalistic Inference of Pathway Activity. Plos One, 2009.
    """

    def __init__(self, **kwargs):
        self.normalize = kwargs.pop("normalize", True) #normalize final results
        super(LLR, self).__init__(**kwargs)

    def build_features(self, *args, **kwargs):
        self._gauss_cache = {} #caching of gaussian estimates
        self._normalizec = {}
        return super(LLR, self).build_features(*args, **kwargs)

    def build_feature(self, data, gs):

        at = Orange.feature.Continuous(name=str(gs))
        geneset = list(gs.genes)

        nm, name_ind, genes, takegenes, to_geneset = self._match_data(data, geneset, odic=True)

        gsi = [ name_ind[g] for g in genes ]
        gausse = compute_llr(data, gsi, self._gauss_cache)
        genes_gs = [ to_geneset[g] for g in genes ]

        if self.normalize: # per (3) in the paper
            #compute log ratios for all samples and genes from this gene set
            for i, gene_gs, g in zip(gsi, genes_gs, gausse):
                if gene_gs not in self._normalizec: #skip if computed already
                    r = [ _llrlogratio(ex[i].value, *g) for ex in data ]
                    self._normalizec[gene_gs] = (statc.mean(r), statc.std(r))

        def t(ex, w, genes_gs=genes_gs, gausse=gausse, normalizec=self._normalizec):
            nm2, name_ind2, genes2 = self._match_instance(ex, genes_gs, None)
            gsvalues = [ vou(ex, gn, name_ind2) for gn in genes2 ]

            vals = [ _llrlogratio(v, *g) if v != "?" else 0.0 for v,g in zip(gsvalues, gausse) ]

            if len(normalizec): #normalize according to (3)
                vals2 = []
                for v,g in zip(vals, genes_gs):
                    m,s = normalizec[g]
                    if s == 0: #disregard attributes without differences
                        vals2.append(0.)
                    else:
                        vals2.append((v-m)/s)
                vals = vals2
            
            return sum(vals)
     
        at.get_value_from = t
        return at

class LLR_slow(ParametrizedTransformation):
    """ Slow and rough implementation of LLR (testing correctness)."""

    def _get_par(self, datao):
        gaussiane = [ estimate_gaussian_per_class(datao, at, common_if_extreme=True) for at in range(len(datao.domain.attributes)) ]
        normalizec = []
        for i,g in zip(range(len(datao.domain.attributes)), gaussiane):
            r = [ _llrlogratio(ex[i].value, *g) for ex in datao ]
            normalizec.append((statc.mean(r), statc.std(r)))
        return gaussiane, normalizec

    def _use_par(self, arr, constructt):
        gaussiane, normalizec = constructt
        arr = [ arr[i].value for i in range(len(arr.domain.attributes)) ]
        return sum ( (_llrlogratio(v, *g)-m)/s for v,g,n in zip(arr, gaussiane, normalizec))


def estimate_linear_fit(data, i):
    """
    Chen et 2008 write about t-score of the least square fit in the
    original article, but here we can use just the t-test, because the
    t-scores obtained are exactly the same (the authors used the
    the more general version as they supported continous outcomes).
    """
    
    """
    #implementation that could also support continous outcomes
    x = numpy.array([ ex[i].value for ex in data ])
    y = numpy.array([ int(ex[-1]) for ex in data ]) #here classes are 0 and 1
    ret = scipy.stats.linregress(x,y)
    b = ret[0]
    seb = ret[4]
    return b/seb
    """
    """
    #per mathword.wolfram.com - results are the same
    c = numpy.cov(x,y)
    n = len(x)
    b = c[0,1]/c[0,0] #the same result as from li
    s = math.sqrt((c[1,1]*n - b*c[0,1]*n)/(n-2))
    seb = s/math.sqrt(c[0,0]*n)
    return b/seb
    """
    return obiExpression.MA_t_test()(i, data)


class SPCA(ParametrizedTransformation):
    """ Per Chen et al. Supervised principal component analysis for
    gene set enrichment of microarray data with continuous or survival
    outcomes. Bioinformatics, 2008.  """

    def __init__(self, **kwargs):
        self.threshold = kwargs.pop("threshold", None)
        self.top = kwargs.pop("top", None)
        self.atleast = kwargs.pop("atleast", 0)
        super(SPCA, self).__init__(**kwargs)

    def _get_par(self, datao):
        scores = [ estimate_linear_fit(datao, a) for a in range(len(datao.domain.attributes)) ]
        scores = list(enumerate(map(abs, scores)))
        select = None
        if self.threshold is not None:
            select = [ i for i,s in scores if s > self.threshold ]
        elif self.top is not None:
            select = nth(sorted(scores, key=lambda x: -x[1])[:self.top], 0)

        if len(select) < self.atleast:
            select = nth(sorted(scores, key=lambda x: -x[1])[:self.atleast], 0)

        if select == None:
            somethingWrongWithSelection

        doms = Orange.data.Domain([ datao.domain.attributes[i] for i in select ], datao.domain.class_var)
        datas = Orange.data.Table(doms, datao)

        if len(select) == 0:
            return select, None
        else:
            return set(select), pca(datas.toNumpy("a")[0])

    def _use_par(self, arr, constructt):
        select, constructt = constructt

        if len(select) == 0:
            return 0.

        arr = [ arr[i].value for i in range(len(arr.domain.attributes)) 
            if i in select ]

        evals, evect, xmean = constructt

        arr = arr - xmean # same input transformation - a row in a matrix
        ev0 = evect[0] #this is a row in a matrix - do a dot product
        a = numpy.dot(arr, ev0)

        return a

def _shuffleClass(data, rand):
    """ Destructive! """
    locations = range(len(data))
    rand.shuffle(locations)
    attribute = -1
    l = [None]*len(data)
    for i in range(len(data)):
        l[locations[i]] = data[i][attribute]
    for i in range(len(data)):
        data[i][attribute] = l[i]

class SPCA_ttperm(SPCA):
    """ Set threshold with a permutation test. """

    def __init__(self, **kwargs):
        self.pval = kwargs.pop("pval", 0.01) #target p-value
        self.perm = kwargs.pop("perm", 100) #number of class permutation
        self.sperm = kwargs.pop("sperm", 100) #sampled attributes per permutation
        super(SPCA_ttperm, self).__init__(**kwargs)

    def build_features(self, data, *args, **kwargs):
        joined = []
        rand = random.Random(0)
        nat = len(data.domain.attributes)

        datap = Orange.data.Table(data.domain, data)

        for p in range(self.perm):
            _shuffleClass(datap, rand)
            if self.sperm is not None:
                ti = rand.sample(xrange(nat), self.sperm)
            else:
                ti = range(nat)
            joined.extend([ obiExpression.MA_t_test()(i, datap) 
                for i in ti ])

        joined = map(abs, joined)
        joined.sort(reverse=True)

        t = joined[int(self.pval*len(joined))]

        self.threshold = t
        return super(SPCA_ttperm, self).build_features(data, *args, **kwargs)
    

if __name__ == "__main__":

    data = Orange.data.Table("iris")

    from .. import gene as obiGene, geneset as obiGeneSets
    

    gsets = obiGeneSets.collections({
        #"ALL": ['sepal length', 'sepal width', 'petal length', 'petal width'],
        "f3": ['sepal length', 'sepal width', 'petal length'],
        "l3": ['sepal width', 'petal length', 'petal width'],
        })
    matcher = obiGene.matcher([])
    choosen_cv = ["Iris-setosa", "Iris-versicolor"]

    """
    data = Orange.data.Table("DLBCL_200a")
    gsets = obiGeneSets.collections((("KEGG",),"9606"))
    matcher = obiGene.matcher([obiGene.GMKEGG("hsa")])
    choosen_cv = None
    """

    def to_old_dic(d, data):
        ar = defaultdict(list)
        for ex1 in data:
            ex = d(ex1)
            for a,x in zip(d.attributes, ex):
                ar[a.name].append(x.value)
        return ar

    def pp2(ar):
        ol =  sorted(ar.items())
        print '\n'.join([ a + ": " +str(b) for a,b in ol])

    #ass = LLR(data, matcher=matcher, gene_sets=gsets, class_values=choosen_cv, min_part=0.0, normalize=True)
    #ass = LLR_slow(data, matcher=matcher, gene_sets=gsets, class_values=choosen_cv, min_part=0.0)
    ass = SetSig(data, matcher=matcher, gene_sets=gsets, class_values=choosen_cv, min_part=0.0, cv=True)
    ar = to_old_dic(ass.domain, data[:5])
    pp2(ar)



"""
<name>Data Files</name>
<description>Reads data from designated directory.</description>
<icon>icons/ChipDataFiles.png</icon>
<priority>1050</priority>
<contact>Peter Juvan (peter.juvan@fri.uni-lj.si)</contact>
<prototype>1</prototype>
"""

import os, os.path

import orange
from Orange.OrangeWidgets.OWWidget import *
from Orange.OrangeWidgets import OWGUI

import warnings
warnings.filterwarnings("ignore", "'strain'", orange.AttributeWarning)

class DataFiles(orange.OrangeBase):
    """Structure for communicating multiple ExampleTables:
    [(name1,[exampleTable1a,exampleTable1b,...]), (name2,[exampleTable2a,...]), ...]
    """
    pass


class ExampleSelection(orange.OrangeBase):
    """Structure for selection of examples: (selectorName, [0,1,...])
    """
    pass


class OWDataFiles(OWWidget):
    settingsList  = ["recentDirs", "selectedDirName", "applyOnChange"]

    def __init__(self, parent=None, signalManager = None, loaddata=1):
        OWWidget.__init__(self, parent, signalManager, 'Data Files', wantMainArea = 0, resizingEnabled = 1)

        self.callbackDeposit = []

        self.inputs = []
        self.outputs = [("Examples", ExampleTable), ("Structured Data", DataFiles)]

        self.dataStructure = []
        self.datasets = None
        self.lastSentIds = []

        # Settings
        self.recentDirs=[]  
        self.selectedDirName = "None"
        self.applyOnChange = 0
        self.loadSettings()

        # CONTROLS
        box = OWGUI.widgetBox(self.controlArea, "Directory", addSpace = True, orientation=0)
        self.dircombo=QComboBox(box)
        box.layout().addWidget(self.dircombo)
        button = OWGUI.button(box, self, '...', callback = self.browseDirectory, disabled=0)
        button.setMaximumWidth(25)
        # connecting GUI to code
        self.connect(self.dircombo,SIGNAL('activated(int)'),self.selectDir)

        # info
        box = OWGUI.widgetBox(self.controlArea, "Info", addSpace = True)
        self.infoa = OWGUI.widgetLabel(box, 'No data loaded.')
        self.infob = OWGUI.widgetLabel(box, '')
        self.infoc = OWGUI.widgetLabel(box, '')
            
        # LIST VIEW
        frmListView = OWGUI.widgetBox(self.controlArea, None, addSpace = True)
        self.tree = QTreeWidget(frmListView)
        self.tree.setSelectionMode(QAbstractItemView.MultiSelection)
        self.tree.setHeaderLabel("Directory/Data File")
        frmListView.layout().addWidget(self.tree)
        self.connect(self.tree,SIGNAL('itemSelectionChanged()'),self.selectionChanged)

        # Output
        box = OWGUI.widgetBox(self.controlArea, "Output", addSpace = True)
        OWGUI.checkBox(box, self, 'applyOnChange', 'Commit data on selection change')
        self.commitBtn = OWGUI.button(box, self, "Commit", callback=self.sendData, disabled=1)
        self.resize(300,600)

        # initial settings            
        self.recentDirs=filter(os.path.exists,self.recentDirs)
        self.setDirlist()
        self.dircombo.setCurrentIndex(0)
        if self.recentDirs!=[] and loaddata:
            self.loadData(self.recentDirs[0])
            
    def setFileTree(self):
        self.disconnect(self.tree,SIGNAL('itemSelectionChanged()'),self.selectionChanged)
        self.tree.clear()
        self.listitems = []
        for d in self.dataStructure:
            (dirname, files) = d
            diritem = QTreeWidgetItem(self.tree, [dirname], QTreeWidgetItem.UserType)
            diritem.setSelected(1)
            self.listitems.append(diritem)
            diritem.setExpanded(1)
            diritem.name = dirname
            for f in files:
                item = QTreeWidgetItem(diritem, [f.name], QTreeWidgetItem.UserType)
                item.setSelected(1)
                self.listitems.append(item)
                item.data = f
        self.connect(self.tree,SIGNAL('itemSelectionChanged()'),self.selectionChanged)
                
    def selectionChanged(self):
        if self.applyOnChange:
            self.sendData()

    # checks which data has been selected, builds a data structure, and sends it out
    def sendData(self):
        data = []
        # clear what has been previously sent
        for id in self.lastSentIds:
            self.send("Examples", None, id)
        self.lastSentIds = []
        # send new data
        id = 0
        for tIdx in range(self.tree.topLevelItemCount()):
            dir = self.tree.topLevelItem(tIdx)
            if dir in self.tree.selectedItems():
                files = []
                for cIdx in range(dir.childCount()):
                    f = dir.child(cIdx)
                    if f in self.tree.selectedItems():
                        files.append(f.data)
                        self.send("Examples", f.data, id)
                        self.lastSentIds.append(id)
                        id += 1
                data.append((dir.name, files))
        #print "DATAS", data
        self.send("Structured Data", data)

    # Loads the data from a root directory, sends the data to the output channels
    def loadData(self, root):
        dataStructure = [] # structured [(dirname0, [d00, d01, ...]), ...]
        datasets = []  # flat list containing all the data sets
        dirs = os.listdir(root)
        lenDirs = len(dirs)
        if lenDirs:
            self.progressBarInit()
            pbStep = 100./lenDirs
        for d in dirs:
            dirname = os.path.join(root, d)
            if os.path.isdir(dirname):
                dirdata = []   
                files  = os.listdir(dirname)
                for f in files:
                    name, ext = os.path.splitext(f)
                    if ext in ['.tab', '.txt', '.data']:
                        try:
                            data = None
                            data = orange.ExampleTable(os.path.join(dirname,f))
                            data.name = name
                            data.strain = os.path.basename(dirname)
                            dirdata.append(data)
                        except orange.KernelException:
                            print 'Warning: file %s\\%s not in appropriate format' %(dirname, f)
                if len(dirdata):
                    dataStructure.append((os.path.split(dirname)[1], dirdata))
                    datasets = datasets + dirdata
            self.progressBarAdvance(pbStep)
        if lenDirs:
            self.progressBarFinished()

        # enable commit, set file tree
        self.commitBtn.setEnabled(len(dataStructure))
        self.dataStructure = dataStructure
        self.datasets = datasets
        self.setFileTree()

        # set infos (sumarize the data)
        if len(dataStructure):
            numSets = len(self.dataStructure)
            numFiles = len(self.datasets)
            self.infoa.setText("%d set%s, total of %d data file%s." % (numSets, ["", "s"][numSets!=1], numFiles, ["","s"][numFiles!=1]))
            # construct lists that sumarize the data
            numExamplesList = []
            numAttrList = []
            hasClass = []
            attrNameList = []
            for et in datasets:
                numExamplesList.append(len(et))
                numAttrList.append(len(et.domain.attributes))
                hasClass.append(et.domain.classVar != None)
                for attr in et.domain.attributes:
                    if attr.name not in attrNameList:
                        attrNameList.append(attr.name)
            # report the number of attributes/class
            if len(numAttrList):
                minNumAttr = min(numAttrList)
                maxNumAttr = max(numAttrList)
                if minNumAttr != maxNumAttr:
                    infob = "From %d to %d attribute%s (%d in total)" % (minNumAttr, maxNumAttr, ["","s"][maxNumAttr!=1], len(attrNameList))
                else:
                    infob = "%d attribute%s" % (maxNumAttr, ["","s"][maxNumAttr!=1])
            else:
                infob = "No attributes"
            if sum(hasClass) == len(hasClass):
                infob += ", all files contain class variable."
            elif sum(hasClass) == 0:
                infob += ", no class variable."
            else:
                infob += ", some files contain class variable."
            self.infob.setText(infob)
            # report the number of examples
            if len(numExamplesList):
                infoc = "Files contain "
                minNumE = min(numExamplesList)
                maxNumE = max(numExamplesList)
                if minNumE == maxNumE:
                    infoc += "%d example%s, " % (maxNumE, ["","s"][maxNumE!=1])
                else:
                    infoc += "from %d to %d example%s, " % (minNumE, maxNumE, ["","s"][maxNumE!=1])
                infoc += "%d in total." % sum(numExamplesList)
            else:
                infoc = "Files contain no examples."
            self.infoc.setText(infoc)

            # Add a directory to the start of the file list. 
            # If it exists, move it to the start of the list
            if root in self.recentDirs:
                self.recentDirs.remove(root)
            self.recentDirs.insert(0, root)
            self.setDirlist()
            self.selectedDirName = root
                
        else:
            self.infoa.setText('No data on input.')
            self.infob.setText('')
            self.infoc.setText('')

        # read data
        if self.applyOnChange:
            self.sendData()

    # displays a file dialog and selects a directory
    def browseDirectory(self):
        if len(self.recentDirs):
            startdir=self.recentDirs[0]
        else:
            startdir ="."
        dirname=str(QFileDialog.getExistingDirectory(self, 'Data Directory', startdir, QFileDialog.ShowDirsOnly))
        if len(dirname):
            self.loadData(str(dirname))

    def setDirlist(self):
        self.dircombo.clear()
        if len(self.recentDirs):
            for dir in self.recentDirs:
                (upperdir,dirname)=os.path.split(dir)
                # leave out the path
                self.dircombo.insertItem(self.dircombo.count(), dirname)
        else:
            self.dircombo.insertItem(0,"(none)")

    # called when user makes a selection from the drop-down menu
    def selectDir(self, n):
        if self.recentDirs:
            self.loadData(self.recentDirs[n])
        else:
            self.loadData("(none)")



if __name__=="__main__":
    a=QApplication(sys.argv)
    ow=OWDataFiles()
    ow.show()
    ow.loadData("/home/marko/anova/smallchipdata")
    a.exec_()
    ow.saveSettings()



"""

"""
from __future__ import absolute_import

KEGG_WDSL = "http://soap.genome.jp/KEGG.wsdl"

REST_API = "http://rest.kegg.jp/"


def slumber_service():
    """
    Return a rest based service using `slumber` package
    """
    import slumber
    if not hasattr(slumber_service, "_cached"):
        slumber_service._cached = slumber.API(REST_API)
    return slumber_service._cached


from . import conf

default_service = slumber_service

web_service = slumber_service



from Orange.bio import go

ontology = go.Ontology()
annotations = go.Annotations("sgd", ontology=ontology)

res = annotations.get_enriched_terms(["YGR270W", "YIL075C", "YDL007W"])
print "Enriched terms:"
for go_id, (genes, p_value, ref) in res.items():
    if p_value < 0.05:
        print ontology[go_id].name, "with p-value: %.4f" % p_value, ", ".join(genes)

# And again for slims
ontology.set_slims_subset("goslim_yeast")

res = annotations.get_enriched_terms(["YGR270W", "YIL075C", "YDL007W"],
                                     slims_only=True)
print "Enriched slim terms:"
for go_id, (genes, p_value, _) in res.items():
    if p_value < 0.05:
        print ontology[go_id].name, "with p-value: %.4f" % p_value, ", ".join(genes)



"""
Backwards compatibility stub. Should be removed by the 2.7 release.
"""
import sys
import warnings

from orangecontrib import bio

# Change the PendingDeprecation to Deprecated in 2.6 release.
warnings.warn(
    "'_bioinformatics' import name is deprecated.\n" +
    "Please use 'orangecontrib.bio'.",
    PendingDeprecationWarning,
    stacklevel=2
)

sys.modules["_bioinformatics"] = bio
